@@ -5,7 +5,7 @@
 //     balloon.c
 //     Creating and Drawing Balloon Message Windows
 //
-//                            Copyright 2002-2007  K.Chimari
+//                            Copyright 2002-2008  K.Chimari
 //                                     http://rosegray.sakura.ne.jp/
 //
 //
@@ -38,21 +38,37 @@ extern GtkWidget *popup_menu;
 extern gint window_x, window_y;
 extern GdkDrawable *pixmap_bal[2];
 
+
 extern gint dw_configure_bal();
+extern gint dw_expose_bal();
 extern void MoveBalloon();
+extern void ResizeMoveBalloon();
+#ifdef USE_CAIRO
+extern gint dw_configure_bal2();
+extern gint dw_expose_bal2();
+extern void screen_changed();
+extern gboolean supports_alpha;
+#endif
 
 extern gboolean flag_balloon;
 
-extern GdkBitmap *mask_bal[2];
+GdkBitmap *mask_bal[2]={NULL,NULL};;
+
+#ifdef USE_WIN32
+  extern void GdkWinSetAlpha();
+#endif
+
+extern gdouble GetCurrentResolution();
 
 gint old_x, old_y;
 
 // *** Protptype of function in this file
 void DrawBalloon();
 void DoBalloon();
-GtkWidget * make_balloon();
+void make_balloon();
+
 #ifdef USE_WIN32
-GtkWidget * make_balloon_fg();
+void make_balloon_fg();
 #endif
 void balloon_arg_init();
 
@@ -82,7 +98,7 @@ void DrawBalloon(typMascot *mascot, char **wn_iwp, int wn_max)
   bal_width=0;
   for(i_wn=0;i_wn<wn_max;i_wn++){
 #ifdef USE_GTK2
-    pango_text=gtk_widget_create_pango_layout(mascot->w_baldraw,
+    pango_text=gtk_widget_create_pango_layout(balloon_main,
 					      wn_iwp[i_wn]);
     pango_layout_get_pixel_size(pango_text,&tmp_w,&tmp_h);
 #else
@@ -163,79 +179,6 @@ void DrawBalloon(typMascot *mascot, char **wn_iwp, int wn_max)
   
     
   // **** pixmapؤBUFFERING
-  // border
-#ifdef USE_WIN32
-  gdk_draw_arc(pixmap_bal[work_page],mascot->gc_balbd[work_page],
-	       TRUE,
-	       0,0,
-	       mascot->baltext_x*2,mascot->baltext_y*2,
-	       0,5760*4);
-  gdk_draw_arc(pixmap_bal[work_page],mascot->gc_balbd[work_page],
-	       TRUE,
-	       (w+2)-mascot->baltext_x*2-1,0,
-	       mascot->baltext_x*2,mascot->baltext_y*2,
-	       0,5760*4);
-  gdk_draw_arc(pixmap_bal[work_page],mascot->gc_balbd[work_page],
-	       TRUE,
-	       0,(h+2)-mascot->baltext_y*2-1,
-	       mascot->baltext_x*2,mascot->baltext_y*2,
-	       0,5760*4);
-  gdk_draw_arc(pixmap_bal[work_page],mascot->gc_balbd[work_page],
-	       TRUE,
-	       (w+2)-mascot->baltext_x*2-1,(h+2)-mascot->baltext_y*2-1,
-	       mascot->baltext_x*2,mascot->baltext_y*2,
-	       0,5760*4);
-#else
-  gdk_draw_arc(pixmap_bal[work_page],mascot->gc_balbd[work_page],
-	       TRUE,
-	       0,0,
-	       mascot->baltext_x*2,mascot->baltext_y*2,
-	       5760,5760);
-  gdk_draw_arc(pixmap_bal[work_page],mascot->gc_balbd[work_page],
-	       TRUE,
-	       w-mascot->baltext_x*2-1,0,
-	       mascot->baltext_x*2,mascot->baltext_y*2,
-	       0,5760);
-  gdk_draw_arc(pixmap_bal[work_page],mascot->gc_balbd[work_page],
-	       TRUE,
-	       0,h-mascot->baltext_y*2-1,
-	       mascot->baltext_x*2,mascot->baltext_y*2,
-	       5760*2,5760);
-  gdk_draw_arc(pixmap_bal[work_page],mascot->gc_balbd[work_page],
-	       TRUE,
-	       w-mascot->baltext_x*2-1,(h+2)-mascot->baltext_y*2-1,
-	       mascot->baltext_x*2,mascot->baltext_y*2,
-	       5760*3,5760);
-#endif
-  gdk_draw_rectangle(pixmap_bal[work_page],mascot->gc_balbd[work_page],
-		     TRUE,
-		     0,mascot->baltext_y,
-		     w,h-mascot->baltext_y*2);
-  gdk_draw_rectangle(pixmap_bal[work_page],mascot->gc_balbd[work_page],
-		     TRUE,
-		     mascot->baltext_x,0,
-		     w-mascot->baltext_x*2,h);
-  
-  points[0].y=h+h_arrow;
-  points[1].y=h-mascot->baltext_y;
-  points[2].y=h-mascot->baltext_y;
-  points[3].y=h+h_arrow;
-  if(mascot->bal_pos==BAL_POS_RIGHT){
-    points[0].x=0;
-    points[1].x=10-mascot->wbalbd;
-    points[2].x=16+mascot->wbalbd;
-    points[3].x=0;
-  }
-  else if(mascot->bal_pos==BAL_POS_LEFT){
-    points[0].x=w;
-    points[1].x=w-10+mascot->wbalbd;
-    points[2].x=w-16-mascot->wbalbd;
-    points[3].x=w;
-  }
-  
-  gdk_draw_polygon(pixmap_bal[work_page],mascot->gc_balbd[work_page],
-		   TRUE,points,4);
-  
   // Mask (BorderƱ)
 #ifdef USE_WIN32
   gdk_draw_arc(mask_bal[work_page],mascot->gc_balmask[work_page],
@@ -288,10 +231,88 @@ void DrawBalloon(typMascot *mascot, char **wn_iwp, int wn_max)
 		     TRUE,
 		     mascot->baltext_x,0,
 		     w-mascot->baltext_x*2,h);
+
+  points[0].y=h+h_arrow;
+  points[1].y=h-mascot->baltext_y;
+  points[2].y=h-mascot->baltext_y;
+  points[3].y=h+h_arrow;
+  if(mascot->bal_pos==BAL_POS_RIGHT){
+    points[0].x=0;
+    points[1].x=10-mascot->wbalbd;
+    points[2].x=16+mascot->wbalbd;
+    points[3].x=0;
+  }
+  else if(mascot->bal_pos==BAL_POS_LEFT){
+    points[0].x=w;
+    points[1].x=w-10+mascot->wbalbd;
+    points[2].x=w-16-mascot->wbalbd;
+    points[3].x=w;
+  }
   gdk_draw_polygon(mask_bal[work_page],mascot->gc_balmask[work_page],
 		   TRUE,points,4);
     
-  // background
+
+  // border
+  if(mascot->wbalbd>0){
+#ifdef USE_WIN32
+    gdk_draw_arc(pixmap_bal[work_page],mascot->gc_balbd[work_page],
+		 TRUE,
+		 0,0,
+		 mascot->baltext_x*2,mascot->baltext_y*2,
+		 0,5760*4);
+    gdk_draw_arc(pixmap_bal[work_page],mascot->gc_balbd[work_page],
+		 TRUE,
+		 (w+2)-mascot->baltext_x*2-1,0,
+		 mascot->baltext_x*2,mascot->baltext_y*2,
+		 0,5760*4);
+    gdk_draw_arc(pixmap_bal[work_page],mascot->gc_balbd[work_page],
+		 TRUE,
+		 0,(h+2)-mascot->baltext_y*2-1,
+		 mascot->baltext_x*2,mascot->baltext_y*2,
+		 0,5760*4);
+    gdk_draw_arc(pixmap_bal[work_page],mascot->gc_balbd[work_page],
+		 TRUE,
+		 (w+2)-mascot->baltext_x*2-1,(h+2)-mascot->baltext_y*2-1,
+		 mascot->baltext_x*2,mascot->baltext_y*2,
+		 0,5760*4);
+#else
+    gdk_draw_arc(pixmap_bal[work_page],mascot->gc_balbd[work_page],
+		 TRUE,
+		 0,0,
+		 mascot->baltext_x*2,mascot->baltext_y*2,
+		 5760,5760);
+    gdk_draw_arc(pixmap_bal[work_page],mascot->gc_balbd[work_page],
+		 TRUE,
+		 w-mascot->baltext_x*2-1,0,
+		 mascot->baltext_x*2,mascot->baltext_y*2,
+		 0,5760);
+    gdk_draw_arc(pixmap_bal[work_page],mascot->gc_balbd[work_page],
+		 TRUE,
+		 0,h-mascot->baltext_y*2-1,
+		 mascot->baltext_x*2,mascot->baltext_y*2,
+		 5760*2,5760);
+    gdk_draw_arc(pixmap_bal[work_page],mascot->gc_balbd[work_page],
+		 TRUE,
+		 w-mascot->baltext_x*2-1,(h+2)-mascot->baltext_y*2-1,
+		 mascot->baltext_x*2,mascot->baltext_y*2,
+		 5760*3,5760);
+#endif
+    gdk_draw_rectangle(pixmap_bal[work_page],mascot->gc_balbd[work_page],
+		       TRUE,
+		       0,mascot->baltext_y,
+		       w,h-mascot->baltext_y*2);
+    gdk_draw_rectangle(pixmap_bal[work_page],mascot->gc_balbd[work_page],
+		       TRUE,
+		       mascot->baltext_x,0,
+		       w-mascot->baltext_x*2,h);
+  
+  
+    gdk_draw_polygon(pixmap_bal[work_page],mascot->gc_balbd[work_page],
+		     TRUE,points,4);
+  }
+  
+
+    // background
 #ifdef USE_WIN32
   gdk_draw_arc(pixmap_bal[work_page],mascot->gc_balbg[work_page],
 	       TRUE,
@@ -372,7 +393,7 @@ void DrawBalloon(typMascot *mascot, char **wn_iwp, int wn_max)
   // Balloon Text
   for(i_wn=0;i_wn<wn_max;i_wn++){
 #ifdef USE_GTK2
-    pango_text=gtk_widget_create_pango_layout(mascot->w_baldraw,
+    pango_text=gtk_widget_create_pango_layout(balloon_main,
 					      wn_iwp[i_wn]);
     
     gdk_draw_layout(pixmap_bal[work_page],
@@ -395,58 +416,45 @@ void DrawBalloon(typMascot *mascot, char **wn_iwp, int wn_max)
   //  Balloon BG Draw  
 #ifdef USE_WIN32
   if((mascot->flag_balfg)&&(mascot->alpha_bal!=100)){
-    gdk_window_set_back_pixmap(mascot->w_baldraw_fg->window,
+    gdk_window_set_back_pixmap(balloon_fg->window,
 			       pixmap_bal[mascot->bal_page],
 			       FALSE);
   }
 #endif
 
-  gdk_window_set_back_pixmap(mascot->w_baldraw->window,
+  gdk_window_set_back_pixmap(balloon_main->window,
 			     pixmap_bal[mascot->bal_page],
 			     FALSE);
 
-  gdk_flush();
-
-
  
   // Resize of Balloon (calling dw_bal_expose)
-#ifdef USE_WIN32
-  if((mascot->flag_balfg)&&(mascot->alpha_bal!=100)){
-    gtk_drawing_area_size (GTK_DRAWING_AREA(mascot->w_baldraw_fg), 
-			   w,
-			   h+h_arrow);
-  }
-#endif
-  gtk_drawing_area_size (GTK_DRAWING_AREA(mascot->w_baldraw), 
-			 w,
-			 h+h_arrow);
-
 #ifdef USE_WIN32
   if((mascot->flag_balfg)&&(mascot->alpha_bal!=100)){
     gdk_window_resize (balloon_fg->window,w,h+h_arrow);
   }
 #endif
   gdk_window_resize (balloon_main->window,w,h+h_arrow);
+  //ResizeMoveBalloon (mascot,mascot->x, mascot->y, w,h+h_arrow);
 
 
 #ifdef USE_WIN32
   if((mascot->flag_balfg)&&(mascot->alpha_bal!=100)){
-    gtk_widget_shape_combine_mask( balloon_fg,
+    gdk_window_shape_combine_mask( balloon_fg->window,
 				   mask_bal[mascot->bal_page],
 				   0,0);
   }
 #endif
 
-  gtk_widget_shape_combine_mask( balloon_main,
+  gdk_window_shape_combine_mask( balloon_main->window,
 				 mask_bal[mascot->bal_page],
 				 0,0);
 
 
   //  End of Balloon Draw  
 
+  MoveBalloon(mascot,mascot->x,mascot->y);
   gdk_flush();
 
-  MoveBalloon(mascot,mascot->x,mascot->y);
 #ifdef USE_WIN32
   if((mascot->flag_balfg)&&(mascot->alpha_bal!=100)){
     gtk_widget_map(balloon_fg);
@@ -457,11 +465,363 @@ void DrawBalloon(typMascot *mascot, char **wn_iwp, int wn_max)
   //while (g_main_iteration(FALSE));
 
 #ifdef USE_GTK2
-  g_object_unref(G_OBJECT(pango_text));
+  if(pango_text)
+    g_object_unref(G_OBJECT(pango_text));
+#endif
+  gdk_flush();
+}
+
+#ifdef USE_CAIRO
+void DrawBalloon2(typMascot *mascot, char **wn_iwp, int wn_max)
+{
+  gint bal_width,bal_height;
+  GdkGCValues gcv;
+  gint w,h;
+  gint lb,rb,as,ds=0;
+  GdkPoint points[4];
+  gint h_arrow=6;
+  int tmp_h,tmp_w,tmp_ds;
+  int i_wn;
+  gint work_page;
+  PangoLayout *pango_text;
+  PangoLanguage *pango_lang;
+  cairo_t *cr;
+  cairo_t *cr_mask;
+  gdouble M_PI=3.14159265;
+  gdouble dx,dy;
+#ifndef __PANGOCAIRO_H__
+  cairo_text_extents_t extents;
+#endif  
+  gboolean shape_flag=FALSE;
+
+  work_page=mascot->bal_page;
+  work_page^=1;
+  
+  if(mascot->flag_composite==COMPOSITE_FALSE){
+    shape_flag=TRUE;
+  }
+  else if((!mascot->force_composite)&&(mascot->flag_composite==COMPOSITE_UNKNOWN)){
+    shape_flag=TRUE;
+  }
+
+  h_arrow+=mascot->wbalbd;
+
+  // bal_height, bal_width  ƥȰԤ礭
+  bal_height=0;
+  bal_width=0;
+  for(i_wn=0;i_wn<wn_max;i_wn++){
+#ifdef __PANGOCAIRO_H__
+    pango_text=gtk_widget_create_pango_layout(balloon_main,
+					      wn_iwp[i_wn]);
+    pango_layout_get_pixel_size(pango_text,&tmp_w,&tmp_h);
+#else
+
+    cr = gdk_cairo_create(balloon_main->window);
+    
+    cairo_select_font_face (cr, 
+			    mascot->fontbal_pc.family,
+			    mascot->fontbal_pc.slant,
+			    mascot->fontbal_pc.weight);
+    cairo_set_font_size (cr, 
+			 mascot->fontbal_pc.pointsize*GetCurrentResolution()/72.0);
+    cairo_text_extents (cr, wn_iwp[i_wn], &extents);
+    tmp_w=(gint)(extents.x_advance);
+    tmp_h=(gint)(extents.height);
+    cairo_destroy(cr);
+#endif
+    
+    if(tmp_w>bal_width){
+      bal_width=tmp_w;
+    }
+     if(tmp_h>bal_height){
+       bal_height=tmp_h;
+       ds=0;
+     }
+  }
+  
+  // w, h Х롼Τ礭
+  w=bal_width+(mascot->baltext_x+mascot->wbalbd)*2;
+  h=(bal_height+mascot->baltext_y)*wn_max
+    +mascot->baltext_y+mascot->wbalbd*2;
+  mascot->balwidth=w;
+  mascot->balheight=h+h_arrow;
+  
+  // Х롼ΰַ
+  mascot->bal_pos=mascot->bal_defpos;
+  if(mascot->bal_pos==BAL_POS_LEFT){
+    if((mascot->x-w+mascot->bal_lxoff[mascot->anime_ptn])<0) 
+      mascot->bal_pos=BAL_POS_RIGHT;
+  }
+  else if(mascot->bal_pos==BAL_POS_RIGHT){
+    if((mascot->x+mascot->width+w+mascot->bal_lxoff[mascot->anime_ptn])
+       >mascot->width_root)
+      mascot->bal_pos=BAL_POS_LEFT;
+  }
+
+    
+  if (pixmap_bal[work_page]) {
+      g_object_unref(G_OBJECT(pixmap_bal[work_page]));
+  } 
+
+ 
+  pixmap_bal[work_page] = gdk_pixmap_new(balloon_main->window,
+					  w,
+					  h+h_arrow,
+					  -1);
+
+  if(shape_flag){
+    if (mask_bal[work_page]) {
+#ifdef USE_GTK2
+      g_object_unref(G_OBJECT(mask_bal[work_page]));
+#else
+      gdk_pixmap_unref(mask_bal[work_page]);
+#endif
+    } 
+    
+    mask_bal[work_page] = gdk_pixmap_new(balloon_main->window,
+					 w,h+h_arrow,1); // Depth =1 (Bitmap)
+    
+    cr_mask = gdk_cairo_create(mask_bal[work_page]);
+    cairo_set_operator (cr_mask, CAIRO_OPERATOR_CLEAR);
+    cairo_paint (cr_mask);
+    cairo_set_operator (cr_mask, CAIRO_OPERATOR_SOURCE);
+    cairo_set_source_rgb (cr_mask, 1, 1, 1); // opaque white
+  }
+
+  cr = gdk_cairo_create(pixmap_bal[work_page]);
+
+  if(shape_flag){
+    cairo_set_source_rgb (cr, 1, 1, 1); // opaque white
+  }
+  else{
+    cairo_set_source_rgba (cr, 1, 1, 1, 0);
+  }
+
+  
+  cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
+  cairo_paint (cr);
+
+  
+  cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
+
+#ifdef USE_WIN32  
+  //BG should be opaque to BG only translucency for Win32
+  cairo_set_source_rgba (cr, 
+			 (gdouble)mascot->colbalbg->red/0xFFFF,
+			 (gdouble)mascot->colbalbg->green/0xFFFF,
+			 (gdouble)mascot->colbalbg->blue/0xFFFF,
+			 1); /* opaque BG */
+  cairo_rectangle(cr, 0, 0, w, h+h_arrow);
+  cairo_fill(cr);
+#else
+  //Even for X, to get clear shape and border,
+  //Full area of the balloon window should be painted with BG color
+  cairo_set_source_rgba (cr, 
+			 (gdouble)mascot->colbalbg->red/0xFFFF,
+			 (gdouble)mascot->colbalbg->green/0xFFFF,
+			 (gdouble)mascot->colbalbg->blue/0xFFFF,
+			 (gdouble)mascot->alpbalbg/0xFFFF); /* transparent */
+  if(shape_flag){
+    cairo_rectangle(cr, 0, 0, w, h+h_arrow);
+    cairo_fill(cr);
+  }
+#endif
+
+    
+  // **** pixmapؤBUFFERING
+  // border
+  dx=(gdouble)mascot->baltext_x;
+  dy=(gdouble)mascot->baltext_y;
+
+  cairo_move_to(cr,0,dy);
+
+  cairo_save (cr);
+  cairo_translate (cr, dx, dy);
+  cairo_scale (cr, dx, dy);
+  cairo_arc (cr, 0, 0, 1., 180*(M_PI/180.), 270*(M_PI/180.));
+  cairo_restore (cr);
+
+  cairo_line_to(cr, w-dx, 0);
+
+  cairo_save (cr);
+  cairo_translate (cr, w-dx, dy);
+  cairo_scale (cr, dx, dy);
+  cairo_arc (cr, 0, 0, 1., 270*(M_PI/180.), 360*(M_PI/180.));
+  cairo_restore (cr);
+
+  cairo_line_to(cr, w, h-dy);
+
+  cairo_save (cr);
+  cairo_translate (cr, w-dx, h-dy);
+  cairo_scale (cr, dx, dy);
+  cairo_arc (cr, 0, 0, 1., 0*(M_PI/180.), 90*(M_PI/180.));
+  cairo_restore (cr);
+  // *** Bottom
+  if(mascot->bal_pos==BAL_POS_RIGHT){
+    cairo_line_to(cr, dx+6+mascot->wbalbd*3,  h);
+    cairo_line_to(cr, 0,  h+h_arrow);
+    cairo_line_to(cr, dx+mascot->wbalbd, h);
+  }
+  else{
+    cairo_line_to(cr, w-dx-mascot->wbalbd,  h);
+    cairo_line_to(cr, w,     h+h_arrow);
+    cairo_line_to(cr, w-dx-6-mascot->wbalbd*3,  h);
+  }
+  cairo_line_to(cr, dx,  h);
+
+  cairo_save (cr);
+  cairo_translate (cr, dx, h-dy);
+  cairo_scale (cr, dx, dy);
+  cairo_arc (cr, 0, 0, 1., 90*(M_PI/180.), 180*(M_PI/180.));
+  cairo_restore (cr);
+
+  cairo_line_to(cr, 0,  dy);
+
+  if(shape_flag){
+    cairo_append_path(cr_mask,cairo_copy_path(cr));
+    cairo_fill(cr_mask);
+    // If having a mask, never clip
+    // Anti-alias of cairo could cause discoloration
+  }
+  else{  
+    cairo_clip_preserve (cr);
+  }
+
+#ifndef USE_WIN32
+  // Paint BG with alpha
+  if(!shape_flag)
+    cairo_fill_preserve(cr);
+#endif
+  
+  if(mascot->wbalbd>0){
+    cairo_set_source_rgba (cr, 
+			   (gdouble)mascot->colbalbd->red/0xFFFF,
+			   (gdouble)mascot->colbalbd->green/0xFFFF,
+			   (gdouble)mascot->colbalbd->blue/0xFFFF,
+			   (gdouble)mascot->alpbalbd/0xFFFF); /* transparent */
+    cairo_set_line_width(cr,(gdouble)mascot->wbalbd *2);
+    cairo_stroke(cr);
+  }
+
+  // Balloon Text
+  cairo_set_source_rgba (cr, 
+			 (gdouble)mascot->colbal->red/0xFFFF,
+			 (gdouble)mascot->colbal->green/0xFFFF,
+			 (gdouble)mascot->colbal->blue/0xFFFF,
+			 (gdouble)mascot->alpbal/0xFFFF); /* transparent */
+
+#ifndef __PANGOCAIRO_H__
+  cairo_select_font_face (cr, 
+			  mascot->fontbal_pc.family,
+			  mascot->fontbal_pc.slant,
+			  mascot->fontbal_pc.weight);
+  cairo_set_font_size (cr, 
+		       mascot->fontbal_pc.pointsize*GetCurrentResolution()/72.0);
+#endif
+
+
+  for(i_wn=0;i_wn<wn_max;i_wn++){
+#ifdef __PANGOCAIRO_H__
+    pango_text=gtk_widget_create_pango_layout(balloon_main,
+					      wn_iwp[i_wn]);
+
+    cairo_move_to(cr,
+		  mascot->baltext_x+mascot->wbalbd,
+		  (mascot->baltext_y+bal_height)*(i_wn+1)
+		  +mascot->wbalbd-bal_height);
+    pango_cairo_show_layout(cr,pango_text);
+#else
+    cairo_text_extents (cr, wn_iwp[i_wn], &extents);
+    cairo_move_to(cr,
+		  mascot->baltext_x+mascot->wbalbd,
+		  -extents.y_bearing+(mascot->baltext_y+bal_height)*(i_wn+1)
+		  +mascot->wbalbd-bal_height);
+    cairo_show_text(cr,wn_iwp[i_wn]);
+#endif
+  }
+
+  cairo_destroy(cr);
+
+  mascot->bal_page=work_page;
+
+ //  Balloon BG Draw  
+#ifdef USE_WIN32
+  if((mascot->flag_balfg)&&(mascot->alpha_bal!=100)){
+    gdk_window_set_back_pixmap(balloon_fg->window,
+			       pixmap_bal[mascot->bal_page],
+			       FALSE);
+  }
+#endif
+
+  gdk_window_set_back_pixmap(balloon_main->window,
+			     pixmap_bal[mascot->bal_page],
+			     FALSE);
+
+ 
+
+#ifdef USE_WIN32
+  if((mascot->flag_balfg)&&(mascot->alpha_bal!=100)){
+    gdk_draw_drawable(balloon_fg->window,
+		      balloon_fg->style->fg_gc[GTK_WIDGET_STATE(balloon_fg)],
+		      pixmap_bal[mascot->bal_page],
+		      0,0,0,0,
+		      w,
+		      h+h_arrow);
+  }
+#endif
+
+  gdk_draw_drawable(balloon_main->window,
+		    balloon_main->style->fg_gc[GTK_WIDGET_STATE(balloon_main)],
+		    pixmap_bal[mascot->bal_page],
+		    0,0,0,0,
+		    w,
+		    h+h_arrow);
+
+  // Resize of Balloon (calling dw_bal_expose)
+#ifdef USE_WIN32
+    if((mascot->flag_balfg)&&(mascot->alpha_bal!=100)){
+      gdk_window_resize (balloon_fg->window,w,h+h_arrow);
+    }
+#endif
+  gdk_window_resize (balloon_main->window,w,h+h_arrow);
+  //ResizeMoveBalloon (mascot,mascot->x, mascot->y, w,h+h_arrow);
+
+  if(shape_flag){
+#ifdef USE_WIN32
+    if((mascot->flag_balfg)&&(mascot->alpha_bal!=100)){
+      gdk_window_shape_combine_mask( balloon_fg->window,
+				     mask_bal[mascot->bal_page],
+				     0,0);
+    }
+#endif
+
+    gdk_window_shape_combine_mask( balloon_main->window,
+				   mask_bal[mascot->bal_page],
+				   0,0);
+    cairo_destroy(cr_mask);
+  }
+
+ 
+  //  End of Balloon Draw  
+
+  MoveBalloon(mascot,mascot->x,mascot->y);
+  gdk_flush();
+#ifdef USE_WIN32
+  if((mascot->flag_balfg)&&(mascot->alpha_bal!=100)){
+    gtk_widget_map(balloon_fg);
+  }
+#endif
+  gtk_widget_map(balloon_main);
+
+#ifdef __PANGOCAIRO_H__
+  if(pango_text)
+    g_object_unref(G_OBJECT(pango_text));
 #endif
   gdk_flush();
 
 }
+#endif
+
 
 
 // Balloonpixmap bufferؤ
@@ -660,7 +1020,14 @@ void DoBalloon(typMascot *mascot)
       balseqp=balseqpend[i_wp];  // ڡֳ֥޿
     }
 
-    DrawBalloon(mascot, wn[i_wp], wn_max[i_wp]);
+    
+#ifdef USE_CAIRO
+    //if((mascot->flag_bal_cairo)&&(mascot->flag_composite!=COMPOSITE_FALSE))
+    if(mascot->flag_bal_cairo)
+      DrawBalloon2(mascot, wn[i_wp], wn_max[i_wp]);
+    else
+#endif
+      DrawBalloon(mascot, wn[i_wp], wn_max[i_wp]);
 
 
     if(i_wp<wp_max-1){
@@ -685,7 +1052,13 @@ void DoBalloon(typMascot *mascot)
   else{
     //ޥåȤưХ롼褹
     if((old_x!=mascot->x)||(old_y!=mascot->y)){
-      DrawBalloon(mascot, wn[i_wp], wn_max[i_wp]);
+#ifdef USE_CAIRO
+      //if((mascot->flag_bal_cairo)&&(mascot->flag_composite!=COMPOSITE_FALSE))
+      if(mascot->flag_bal_cairo)
+	DrawBalloon2(mascot, wn[i_wp], wn_max[i_wp]);
+      else
+#endif
+	DrawBalloon(mascot, wn[i_wp], wn_max[i_wp]);
     }
 
     // 󥿤򸺤餹
@@ -722,6 +1095,7 @@ void DoBalloon(typMascot *mascot)
 	gtk_widget_unmap(balloon_fg);
 #endif
 	gtk_widget_unmap(balloon_main);
+
 	flag_balloon=FALSE;
       }
       else{
@@ -735,12 +1109,7 @@ void DoBalloon(typMascot *mascot)
 }
 
 
-
-
-GtkWidget * make_balloon(void){
-  GtkWidget *balloon_vbox;
-  GtkWidget *balloon_drawing_area;
-
+void make_balloon(typMascot *mascot){
   balloon_main = gtk_window_new(GTK_WINDOW_POPUP);
 #ifdef USE_GTK2
   gtk_window_set_accept_focus(GTK_WINDOW(balloon_main),FALSE);
@@ -756,30 +1125,29 @@ GtkWidget * make_balloon(void){
 			GDK_EXPOSURE_MASK);
   gtk_widget_realize(balloon_main);
   gdk_window_set_decorations(balloon_main->window, 0);
+#ifdef USE_GTK2
+  gtk_window_set_resizable(GTK_WINDOW(balloon_main),TRUE);
+#endif
 #ifndef USE_WIN32
   /* gdk_window_set_override_redirect is not implemented (for warning) */
   gdk_window_set_override_redirect(balloon_main->window,TRUE);
 #endif
 
-  balloon_vbox = gtk_hbox_new(FALSE, 0);
-  gtk_container_add(GTK_CONTAINER (balloon_main), balloon_vbox);
-  gtk_widget_show(balloon_vbox);
 
-  balloon_drawing_area = gtk_drawing_area_new();
-  gtk_drawing_area_size (GTK_DRAWING_AREA(balloon_drawing_area), 1, 1);
-  gtk_box_pack_start(GTK_BOX(balloon_vbox), balloon_drawing_area,
-		     TRUE, TRUE, 0);
-  gtk_widget_show(balloon_drawing_area);
+  my_signal_connect(balloon_main, "configure_event",
+		    dw_configure_bal, (gpointer)mascot);
+  my_signal_connect(balloon_main, "expose_event",
+		    dw_expose_bal, (gpointer)mascot);
 
-
-
-  return(balloon_drawing_area);
+#ifdef USE_GTK2
+  gtk_window_resize (GTK_WINDOW(balloon_main),1,1);
+#endif
+  //gtk_widget_show(balloon_main);
 }
 
+
 #ifdef USE_WIN32
-GtkWidget * make_balloon_fg(void){
-  GtkWidget *balloon_vbox;
-  GtkWidget *balloon_drawing_area;
+void make_balloon_fg(typMascot *mascot){
 
   balloon_fg = gtk_window_new(GTK_WINDOW_POPUP);
 #ifdef USE_GTK2
@@ -796,20 +1164,15 @@ GtkWidget * make_balloon_fg(void){
 			GDK_EXPOSURE_MASK);
   gtk_widget_realize(balloon_fg);
   gdk_window_set_decorations(balloon_fg->window, 0);
+  gtk_window_set_resizable(GTK_WINDOW(balloon_main),TRUE);
 
-  balloon_vbox = gtk_hbox_new(FALSE, 0);
-  gtk_container_add(GTK_CONTAINER (balloon_fg), balloon_vbox);
-  gtk_widget_show(balloon_vbox);
+  my_signal_connect(balloon_fg, "configure_event",
+		    dw_configure_bal, (gpointer)mascot);
+  my_signal_connect(balloon_fg, "expose_event",
+		    dw_expose_bal, (gpointer)mascot);
 
-  balloon_drawing_area = gtk_drawing_area_new();
-  gtk_drawing_area_size (GTK_DRAWING_AREA(balloon_drawing_area), 1, 1);
-  gtk_box_pack_start(GTK_BOX(balloon_vbox), balloon_drawing_area,
-		     TRUE, TRUE, 0);
-  gtk_widget_show(balloon_drawing_area);
-
-
-
-  return(balloon_drawing_area);
+  gtk_window_resize (GTK_WINDOW(balloon_fg),1,1);
+  //gtk_widget_show(balloon_fg);
 }
 #endif
 
