@@ -5,7 +5,7 @@
 //     callback.c
 //     Callback functions
 //
-//                            Copyright 2002-2007  K.Chimari
+//                            Copyright 2002-2008  K.Chimari
 //                                     http://rosegray.sakura.ne.jp/
 //
 //
@@ -38,6 +38,7 @@
 #include <mmsystem.h>
 #else // for WIN32
 #include <gdk/gdkx.h>
+#include <X11/Xatom.h>
 #endif
 
 #include "sockmsg.h"
@@ -47,16 +48,19 @@
 // *** GLOBAL ARGUMENT ***
 extern GtkWidget *win_main,*balloon_main,*clock_main;
 #ifdef USE_WIN32
-extern GtkWidget *clock_fg, *balloon_fg;
+extern GtkWidget *win_sdw, *clock_fg, *balloon_fg;
 #endif
 #ifdef USE_BIFF
 extern GtkWidget *biff_pix;
 #endif
+#ifndef __GTK_TOOLTIP_H__
 extern GtkTooltips *tooltip;
+#endif
 extern GtkWidget *PopupMenu; 
 extern GdkDrawable *pixmap_main[2], *pixmap_clk[2], *pixmap_bal[2];
-
-extern GdkBitmap *mask_bal[2];
+#ifdef USE_WIN32
+extern GdkDrawable *pixmap_sdw[2];
+#endif
 
 extern gchar *FullPathSoundFile();
 
@@ -75,6 +79,8 @@ extern void display_biff_balloon();
 extern void copy_file();
 extern gboolean my_main_iteration();
 
+void DrawPanelClock0();
+
 
 // *** GLOBAL ARGUMENT in this code *** 
 int pre_hour, pre_min, pre_sec;
@@ -111,6 +117,7 @@ void MoveMascot();
 void MoveBiffPix();
 #endif
 void MoveBalloon();
+void ResizeMoveBalloon();
 int MoveToFocus();
 
 GdkGC *MPCreatePen();
@@ -125,6 +132,11 @@ void biff_drag_end();
 #endif
 
 gint dw_configure_main();
+gint dw_init_main();
+#ifdef USE_WIN32
+gint dw_configure_sdw();
+gint dw_init_sdw();
+#endif
 #ifdef USE_BIFF
 gint dw_configure_biff_pix();
 #endif
@@ -143,9 +155,8 @@ void biff_window_motion();
 #endif
 void focus_in();
 void focus_out();
-gint time_update();
+gboolean time_update();
 void clock_update();
-void DrawPanelClock();
 gchar * ini_week();
 
 void ext_play();
@@ -159,11 +170,15 @@ void raise_all();
 void callbacks_arg_init();
 
 #ifdef USE_WIN32
-int GetTaskbarHeight( );
-int GetTaskbarWidth( );
+MyXY GetTaskbarHeight( );
 gboolean IsWin32Root();
 #endif
 
+#ifndef USE_WIN32
+gchar* Get_Window_Name();
+gint Get_Window_Bar_Size();
+MyXY GetAutoHomePos();
+#endif
 
 // ŤߤĤѥ
 int weight_random(typMascot *mascot)
@@ -212,6 +227,19 @@ void MoveMascot(typMascot *mascot, gint x, gint y)
   mascot->x=x;
   mascot->y=y;
 
+#ifdef USE_WIN32
+  if((mascot->sdw_flag)&&(mascot->sdw_height>0)){
+    if(mascot->move==MOVE_FOCUS){
+      gdk_window_move(win_sdw->window, 
+		      x+mascot->sdw_x, 
+		      y+mascot->height-mascot->yoff*((gfloat)(mascot->magnify)/100));
+    }
+    else{
+      gdk_window_move(win_sdw->window, x+mascot->sdw_x, y+mascot->sdw_y);
+    }
+  }
+#endif
+
   gdk_window_move(win_main->window, x, y);
 
   MoveBalloon(mascot, x, y);
@@ -224,13 +252,19 @@ void MoveMascot(typMascot *mascot, gint x, gint y)
     gdk_window_move(clock_main->window, 
 		    x+mascot->clk_x*((gfloat)(mascot->magnify)/100), 
 		    y+mascot->clk_y*((gfloat)(mascot->magnify)/100));
-  }	 
+  }
 #ifdef USE_BIFF
   MoveBiffPix(mascot, x, y);
 #endif
   
   raise_all();
 
+  gdk_window_move(win_main->window, x, y);
+
+#ifdef USE_WIN32
+  gtk_menu_popdown(GTK_MENU(PopupMenu));
+#endif
+
   gdk_flush();
 
 }
@@ -291,6 +325,41 @@ void MoveBalloon(typMascot *mascot, gint x, gint y)
  gdk_flush();
 }
 
+void ResizeMoveBalloon(typMascot *mascot, gint x, gint y, gint w, gint h)
+{
+  gint bal_y;
+  bal_y=y+mascot->bal_lyoff[mascot->anime_ptn];
+
+  if(bal_y<0) bal_y=0;
+
+  if(mascot->bal_pos==BAL_POS_LEFT){
+#ifdef USE_WIN32
+    gdk_window_move_resize(balloon_fg->window, 
+			   x-w+mascot->bal_lxoff[mascot->anime_ptn],
+			   bal_y,
+			   w,h);
+#endif
+    gdk_window_move_resize(balloon_main->window, 
+			   x-w+mascot->bal_lxoff[mascot->anime_ptn],
+			   bal_y,
+			   w,h);
+  }
+  else{
+#ifdef USE_WIN32
+    gdk_window_move_resize(balloon_fg->window, 
+			   x+mascot->width+mascot->bal_rxoff[mascot->anime_ptn],
+			   bal_y,
+			   w,h);
+#endif
+    gdk_window_move_resize(balloon_main->window, 
+			   x+mascot->width+mascot->bal_rxoff[mascot->anime_ptn],
+			   bal_y,
+			   w,h);
+  }
+
+ gdk_flush();
+}
+
 // եWindowɽư
 int MoveToFocus(typMascot *mascot, gboolean force_fl)
 {
@@ -310,6 +379,7 @@ int MoveToFocus(typMascot *mascot, gboolean force_fl)
   unsigned int nchildren;
   unsigned int width,height,width_root,height_root;
   int rx,ry,wx,wy,mask_return,realXPos;
+  gint win_bar_size;
 #endif
   gboolean eflag=FALSE;
   gint flag_homepos=HOMEPOS_NEVER;
@@ -330,11 +400,13 @@ int MoveToFocus(typMascot *mascot, gboolean force_fl)
       DWORD wStyle;
       wStyle = GetWindowLong (hWnd,GWL_STYLE);
       if(!(wStyle&WS_CAPTION)){
+	flag_homepos=mascot->homepos_nb;
 	hWnd=hWnd_active;
       }
     }
   }
 #else
+  win_bar_size=0;
   XGetInputFocus(GDK_WINDOW_XDISPLAY(win_main->window),
 		 &wf, &i);
   XGetGeometry(GDK_WINDOW_XDISPLAY(win_main->window),
@@ -381,7 +453,7 @@ int MoveToFocus(typMascot *mascot, gboolean force_fl)
     sy = nRect.top;
     width = nRect.right-nRect.left;
 
-#else
+#else  // UNIX X
   if(wf!=GDK_WINDOW_XWINDOW(win_main->window) 
      && wf!=GDK_WINDOW_XWINDOW(balloon_main->window)
      && wf!=GDK_WINDOW_XWINDOW(clock_main->window)
@@ -390,7 +462,8 @@ int MoveToFocus(typMascot *mascot, gboolean force_fl)
 #endif
      && (int)wf!=0){
 
-    if(mascot->focus_autobar==TRUE){ // ȥСΥư 
+    if(mascot->focus_autobar!=AUTOBAR_MANUAL){ 
+      // ȥСΥư 
       if(errflag ==0){
 	XQueryTree(GDK_WINDOW_XDISPLAY(win_main->window),
 		   wf,&rootwin,&parent,&children,&nchildren);
@@ -418,21 +491,71 @@ int MoveToFocus(typMascot *mascot, gboolean force_fl)
 		       &border,&depth);
 	}
 	else{
-	  errflag = 0;
+	  errflag   = 0;
 	  return(0);
 	}
       }
       else{
+	//Window wf_org;
+
 	for(;;){
-	  if(parent == rootwin)
+	  if(parent == rootwin){
+	    /*
+	    if(mascot->focus_autobar==AUTOBAR_ORDINAL){
+	      if(win_bar_size==0){  
+		win_bar_size
+		  =Get_Window_Bar_Size(GDK_WINDOW_XDISPLAY(win_main->window),
+				       wf_org);
+		if((win_bar_size==0)&&(mascot->no_capbar)){
+		  // for windows w/o title bar 
+		  flag_homepos=mascot->homepos_nb;
+		  if(flag_homepos==HOMEPOS_NEVER) return;
+		}
+	      }
+	      //win_bar_size=0;
+	    }
+	    else {  // for Compiz 
+	      if(win_bar_size==0){  
+		win_bar_size
+		  =Get_Window_Bar_Size(GDK_WINDOW_XDISPLAY(win_main->window),
+				       wf);
+		if((win_bar_size==0)&&(mascot->no_capbar)){
+		  // for windows w/o title bar 
+		  flag_homepos=mascot->homepos_nb;
+		  if(flag_homepos==HOMEPOS_NEVER) return;
+		}
+	      }
+	    }
 	    break;
+	    */
+	    
+	    if(win_bar_size==0){
+	      win_bar_size
+		=Get_Window_Bar_Size(GDK_WINDOW_XDISPLAY(win_main->window),
+				     wf);
+	    }
+
+	    if((win_bar_size==0)&&(mascot->no_capbar)){
+	      // for windows w/o title bar 
+	      flag_homepos=mascot->homepos_nb;
+	      if(flag_homepos==HOMEPOS_NEVER) return;
+	    }
+	    break;
+	  }
 	  XGetGeometry(GDK_WINDOW_XDISPLAY(win_main->window),
 		       parent,&rootwin,&x_root,&y_root,
 		       &width,&height,&border,&depth);
+	     
 	  if((width==width_root)&&(height==height_root)){
 	    eflag=TRUE;
 	    break; // For Enlightenment
 	  }
+
+	  if(win_bar_size==0){
+	    win_bar_size
+	      =Get_Window_Bar_Size(GDK_WINDOW_XDISPLAY(win_main->window),
+				   wf);
+	  }
 	  
 	  if(mascot->bar_offset!=0){
 	    if(errflag ==0){
@@ -445,11 +568,13 @@ int MoveToFocus(typMascot *mascot, gboolean force_fl)
 	      return(0);
 	    }
 	  }
-	  
+
+	  //wf_org = wf;
 	  wf = parent;
+
 	  if(errflag ==0){
 	    XQueryTree(GDK_WINDOW_XDISPLAY(win_main->window),
-		       wf,&rootwin,&parent,&children,&nchildren);
+	    	       wf,&rootwin,&parent,&children,&nchildren);
 	    if(errflag ==0)  XFree(children);
 	  }
 	  else{
@@ -462,8 +587,8 @@ int MoveToFocus(typMascot *mascot, gboolean force_fl)
 
 	if(errflag ==0){
 	  XGetGeometry(GDK_WINDOW_XDISPLAY(win_main->window),
-		       wf,&rootwin,&x,&y,&width,&height,
-		       &border,&depth);
+	  	       wf,&rootwin,&x,&y,&width,&height,
+	  	       &border,&depth);
 	  // եWindow¸ߤʤ Offset"%"ΤȤ
 	  //		if (mascot->flag_xp){
 	  // if((x<0)&&(y<0)&&(depth==0)&&(border==0)){
@@ -478,6 +603,7 @@ int MoveToFocus(typMascot *mascot, gboolean force_fl)
       }
 
       sx=x;
+#ifdef USE_WIN32
       if(mascot->bar_offset==0){
 	sy=y;
       }
@@ -487,6 +613,18 @@ int MoveToFocus(typMascot *mascot, gboolean force_fl)
       else{    // եåȷ¬ : ޥåȤϥСͭWin˾äƤ
 	sy=y-mascot->bar_offset;
       }
+#else
+      if(mascot->focus_autobar==AUTOBAR_COMPIZ){
+	y=y-win_bar_size;
+      }
+
+      if((mascot->bar_offset!=0)&&(win_bar_size!=0)){
+	sy=y-mascot->bar_offset;
+      }
+      else{
+	sy=y;
+      }
+#endif
     }
     else{  // ȥСΥޥ˥奢⡼
       if(errflag ==0){
@@ -498,6 +636,20 @@ int MoveToFocus(typMascot *mascot, gboolean force_fl)
 	return(0);
       }
 
+      if(width==1){  // ľ Focused WindowΥȤƤʤ
+	XQueryTree(GDK_WINDOW_XDISPLAY(win_main->window),
+		   wf,&rootwin,&parent,&children,&nchildren);
+	wf=parent;
+	if(errflag ==0){
+	  XGetGeometry(GDK_WINDOW_XDISPLAY(win_main->window),
+		       wf,&rootwin,&x,&y,&width,&height,&border,&depth);
+	}
+	else{
+	  errflag = 0;
+	  return(0);
+	}
+      }
+
       if(errflag ==0){
 	XQueryPointer(GDK_WINDOW_XDISPLAY(win_main->window),
 		      wf, &rootwin, &child, &rx, &ry,
@@ -508,9 +660,11 @@ int MoveToFocus(typMascot *mascot, gboolean force_fl)
 	return(0);
 	
       }
+
+
       sx=rx-wx;
       sy=ry-wy-mascot->bar_size;
-    }
+      }
 #endif
 
     // sx, sy ϸߤΥեWindowκ(ޥȥС)а
@@ -541,14 +695,26 @@ int MoveToFocus(typMascot *mascot, gboolean force_fl)
     
     if(eflag){
       xpop=sx+realXPos+x_root;
-      ypop=sy+mascot->yoff*((gfloat)(mascot->magnify)/100)+y_root-mascot->height;
+#ifdef USE_CAIRO
+      ypop=sy+mascot->yoff*((gfloat)(mascot->magnify)/100)
+	+y_root-mascot->height+mascot->sdw_y_int;
+#else
+      ypop=sy+mascot->yoff*((gfloat)(mascot->magnify)/100)
+	+y_root-mascot->height;
+#endif
     }
     else{
       xpop=sx+realXPos;
-      ypop=sy+mascot->yoff*((gfloat)(mascot->magnify)/100)-mascot->height;
+#ifdef USE_CAIRO
+      ypop=sy+mascot->yoff*((gfloat)(mascot->magnify)/100)
+	-mascot->height+mascot->sdw_y_int;
+#else
+      ypop=sy+mascot->yoff*((gfloat)(mascot->magnify)/100)
+	-mascot->height;
+#endif
     }
 
-    if(mascot->focus_autobar){
+    if(mascot->focus_autobar!=AUTOBAR_MANUAL){
 #ifdef USE_WIN32   // եʤ 
       if(IsWin32Root(hWnd, mascot->width_root, mascot->height_root)){
 #else
@@ -573,7 +739,7 @@ int MoveToFocus(typMascot *mascot, gboolean force_fl)
 	flag_homepos=mascot->homepos_out;
       }
     }
-   
+       
     switch(flag_homepos){
     case HOMEPOS_BAR:
       if(mascot->ff_side==FF_SIDE_RIGHT){
@@ -596,11 +762,47 @@ int MoveToFocus(typMascot *mascot, gboolean force_fl)
 	}
       }
 #ifdef USE_WIN32
-      mascot->home_y=GetTaskbarHeight();
-      mascot->home_x=mascot->width_root-GetTaskbarWidth();
-#endif      
+      {
+	MyXY xy;
+
+	xy=GetTaskbarHeight();
+	
+	if(xy.flag){
+	  mascot->home_x=mascot->width_root-xy.x;
+	  mascot->home_y=xy.y;
+	}
+	if((!xy.flag)||(mascot->home_y==0)){
+	  if(mascot->home_auto_vanish){
+	    ypop=mascot->height_root;
+	    break;
+	  }
+	}
+      }
+#else
+      if(mascot->home_auto){  // Auto Taskbar Scaling for X
+	MyXY xy;
+
+	xy=GetAutoHomePos();
+	if(xy.flag){
+	  mascot->home_x=mascot->width_root-xy.x;
+	  mascot->home_y=mascot->height_root-xy.y;
+	}
+	if((!xy.flag)||(mascot->home_y==0)){
+	  if(mascot->home_auto_vanish){
+	    ypop=mascot->height_root;
+	    break;
+	  }
+	}
+      }
+#endif
+#ifdef USE_CAIRO
+      ypop=(mascot->height_root-mascot->home_y)
+	+mascot->yoff*((gfloat)(mascot->magnify)/100)-mascot->height
+	+mascot->sdw_y_int;
+#else
       ypop=(mascot->height_root-mascot->home_y)
 	+mascot->yoff*((gfloat)(mascot->magnify)/100)-mascot->height;
+#endif
       break;
       
     case HOMEPOS_VANISH:
@@ -634,51 +836,7 @@ int MoveToFocus(typMascot *mascot, gboolean force_fl)
       }
     }
   }
-      /*
-#ifdef USE_WIN32
-  else if(mascot->task_force){
-    // Stationed on Task Bar for Windows
-    if(mascot->task_force){
-      oxpop=xpop;
-      oypop=ypop;
-      
-      
-      if(mascot->ff_side==FF_SIDE_RIGHT){
-	if(mascot->flag_xp){
-	  xpop=((mascot->width_root-mascot->home_x)-mascot->width)
-	    *(1-(mascot->offsetp*0.01)) -mascot->xoff;
-	}
-	else{
-	  xpop=(mascot->width_root-mascot->home_x)
-	    -mascot->width-mascot->xoff-mascot->offset;
-	}
-      }
-      else{
-	if(mascot->flag_xp){
-	  xpop=((mascot->width_root-mascot->home_x)-mascot->width)
-	    *(mascot->offsetp*0.01) -mascot->xoff;
-	}
-	else{
-	  xpop=mascot->home_x-mascot->xoff+mascot->offset;
-	}
-      }
-      
-      mascot->home_y=GetTaskbarHeight();
-      mascot->home_x=mascot->width_root-GetTaskbarWidth();
-      
-      ypop=(mascot->height_root-mascot->home_y)
-	+mascot->yoff*((gfloat)(mascot->magnify)/100)-mascot->height;
-      
-      if((oxpop!=xpop)||(oypop!=ypop)){
-	MoveMascot(mascot, xpop, ypop);
-	
-	FWinWidth=mascot->width_root-mascot->home_x;
-	FWinX=0;
-      }
-    }
-  }
-#endif
-      */
+
   while (my_main_iteration(FALSE));
   gdk_flush();
   return(1);
@@ -693,11 +851,15 @@ GdkGC *MPCreatePen(GdkDrawable *pixmap, GdkColor *color)
 
   gdk_colormap_alloc_color(gdk_colormap_get_system(),color,FALSE,TRUE);
 
-  gc=gdk_gc_new(pixmap);
+  if(pixmap){
+    gc=gdk_gc_new(pixmap);
   
-  gdk_gc_set_foreground(gc, color);
-
-  return(gc);
+    gdk_gc_set_foreground(gc, color);
+    return(gc);
+  }
+  else{
+    return(NULL);
+  }
 }
 
 
@@ -903,29 +1065,48 @@ gint dw_configure_main(GtkWidget *widget, GdkEventConfigure *event,
   printf("Configure: \n");
 #endif
 
-  if (pixmap_main[0]) {
-#ifdef USE_GTK2
-    g_object_unref(G_OBJECT(pixmap_main[0]));
-#else
-    gdk_pixmap_unref(pixmap_main[0]);
+  //DrawMascot0(mascot);
+  gdk_window_set_back_pixmap(win_main->window,
+			     pixmap_main[mascot->pixmap_page],
+			     FALSE);
+
+#ifdef DEBUG
+  printf("End of Configure: \n");
 #endif
-  } 
-  if (pixmap_main[1]) {
-#ifdef USE_GTK2
-    g_object_unref(G_OBJECT(pixmap_main[1]));
-#else
-    gdk_pixmap_unref(pixmap_main[1]);
+  return FALSE;
+}
+
+#ifdef USE_WIN32
+gint dw_configure_sdw(GtkWidget *widget, GdkEventConfigure *event,
+		       gpointer gdata)
+{
+  typMascot *mascot;
+  mascot=(typMascot *)gdata; 
+
+  if((mascot->sdw_flag)&&(mascot->sdw_height>0)){
+    gdk_window_set_back_pixmap(win_sdw->window,
+			       pixmap_sdw[mascot->pixmap_page],
+			       FALSE);
+  }
+  return FALSE;
+}
 #endif
-  } 
-  
-  pixmap_main[0] = gdk_pixmap_new(widget->window,
-			       widget->allocation.width,
-			       widget->allocation.height,
-			       -1);
-  pixmap_main[1] = gdk_pixmap_new(widget->window,
-			       widget->allocation.width,
-			       widget->allocation.height,
-			       -1);
+
+gint dw_init_main(GtkWidget *widget, GdkEventConfigure *event,
+		  gpointer gdata)
+{
+  typMascot *mascot;
+  mascot=(typMascot *)gdata; 
+  gint i_work;
+  gint i_frm=0;
+
+#ifdef DEBUG
+  printf("Init Main: \n");
+#endif
+
+  if(mascot->anime_frm==-1) i_frm=0;
+  else i_frm=mascot->anime_frm;
+
   if(mascot->gc_main[0]==NULL){
     mascot->gc_main[0] = MPCreatePen(pixmap_main[0], mascot->colclk);
     mascot->gc_mainsd[0] = MPCreatePen(pixmap_main[0], mascot->colclksd);
@@ -937,18 +1118,47 @@ gint dw_configure_main(GtkWidget *widget, GdkEventConfigure *event,
     if(errflag==-1){
       XSetErrorHandler (ehandler);
     }
+#endif
+#ifdef DEBUG
+    printf("Init GC Main\n");
 #endif
   }
-  DrawMascot0(mascot);
+  
+  for(i_work=0;i_work<2;i_work++){
+#ifdef USE_GTK2
+    gdk_draw_drawable(pixmap_main[i_work],
+		      mascot->gc_main[i_work],
+		      mascot->sprites[mascot->frame_pix[0][0]].pixmap,
+		      0, 0,
+		      0, 0,
+		      mascot->width, mascot->height);
+#else
+    gdk_draw_pixmap(pixmap_main[i_work],
+		    mascot->gc_main[i_work],
+		    mascot->sprites[mascot->frame_pix[0][0]].pixmap,
+		    0, 0,
+		    0, 0,
+		    mascot->width, mascot->height);
+#endif
+  }
+  
+  //DrawMascot0(mascot);
+  gdk_window_set_back_pixmap(win_main->window,
+			     pixmap_main[mascot->pixmap_page],
+			     FALSE);
 
 #ifdef USE_WIN32  // Get Handle of Task-Bar
   hWndTaskBar = FindWindow("Shell_TrayWnd", NULL);
 #endif  
 
-  return TRUE;
+#ifdef DEBUG
+  printf("End of Init Main: \n");
+#endif
+  return FALSE;
 }
 
 
+
 #ifdef USE_BIFF
 // Biff Pixmap  WindowѷˤȤʤCallback
 gint dw_configure_biff_pix(GtkWidget *widget, GdkEventConfigure *event,
@@ -967,7 +1177,7 @@ gint dw_configure_biff_pix(GtkWidget *widget, GdkEventConfigure *event,
 		  0,0,0,0,
 		  widget->allocation.width,
 		  widget->allocation.height);
-  return TRUE;
+  return FALSE;
 }
 #endif
 
@@ -992,7 +1202,6 @@ gint dw_configure_clk(GtkWidget *widget, GdkEventConfigure *event,
   }
 
 
-
   if(mascot->gc_clk[0]==NULL){
     mascot->gc_clk[0] = MPCreatePen(pixmap_clk[0], mascot->colclk);
     mascot->gc_clksd[0] = MPCreatePen(pixmap_clk[0], mascot->colclksd);
@@ -1011,17 +1220,17 @@ gint dw_configure_clk(GtkWidget *widget, GdkEventConfigure *event,
     // **** windowؤ
 #ifdef USE_WIN32
     if((mascot->flag_clkfg)&&(mascot->alpha_clk!=100)){
-      gdk_window_set_back_pixmap(mascot->w_clkdraw_fg->window,
+      gdk_window_set_back_pixmap(clock_fg->window,
 				 pixmap_clk[mascot->clk_page],
 				 FALSE);
     }
 #endif
-    gdk_window_set_back_pixmap(mascot->w_clkdraw->window,
-    			       pixmap_clk[mascot->clk_page],
-			       FALSE);
+      gdk_window_set_back_pixmap(clock_main->window,
+				 pixmap_clk[mascot->clk_page],
+				 FALSE);
   }
-  return TRUE;
-}
+  return FALSE;
+ }
 
 
 // Balloon WindowѷˤȤʤCallback
@@ -1056,7 +1265,8 @@ gint dw_configure_bal(GtkWidget *widget, GdkEventConfigure *event,
     mascot->gc_balbg[1] = MPCreatePen(pixmap_bal[1], mascot->colbalbg);
     mascot->gc_balbd[1] = MPCreatePen(pixmap_bal[1], mascot->colbalbd);
   }
-  return TRUE;
+
+  return FALSE;
 }
 
 
@@ -1089,6 +1299,25 @@ gint dw_expose_main(GtkWidget *widget, GdkEventExpose *event,  gpointer gdata)
   return FALSE;
 }
 
+#ifdef USE_WIN32
+gint dw_expose_sdw(GtkWidget *widget, GdkEventExpose *event,  gpointer gdata)
+{
+  typMascot *mascot;
+
+  mascot=(typMascot *)gdata; 
+  
+  // **** Mascot windowؤ
+  if((mascot->sdw_flag)&&(mascot->sdw_height>0))
+    gdk_draw_drawable(widget->window,
+		      widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
+		      pixmap_sdw[mascot->pixmap_page],
+		      event->area.x, event->area.y,
+		      event->area.x, event->area.y,
+		      event->area.width, event->area.height);
+
+  return FALSE;
+}
+#endif
 
 #ifdef USE_BIFF
 // Biff Pixmap Ťʤν
@@ -1128,22 +1357,21 @@ gint dw_expose_clk(GtkWidget *widget, GdkEventExpose *event,  gpointer gdata)
     // **** Mascot windowؤ
     // **** windowؤ
 #ifdef USE_GTK2
-    gdk_draw_drawable(widget->window,
+      gdk_draw_drawable(widget->window,
 #else
-    gdk_draw_pixmap(widget->window,
+      gdk_draw_pixmap(widget->window,
 #endif
-		    mascot->w_clkdraw->style->fg_gc[GTK_WIDGET_STATE (widget)],
+		    clock_main->style->fg_gc[GTK_WIDGET_STATE (widget)],
 		    pixmap_clk[mascot->clk_page],
 		    0,0,0,0,
 		    widget->allocation.width,
 		    widget->allocation.height);
-
   }
-
   return FALSE;
 }
 
 
+
 // Balloon Ťʤν
 // WorkƤʤBufferPixmapԤ
 // 1.6.1Ǽ¼ drawballoonΤʤ back_pixmapꤹΤߤȤ
@@ -1154,19 +1382,20 @@ gint dw_expose_bal(GtkWidget *widget, GdkEventExpose *event,  gpointer gdata)
   
   // **** Balloon windowؤ
 
+
 #ifdef USE_GTK2
-  gdk_draw_drawable(widget->window,
+    gdk_draw_drawable
 #else
-  gdk_draw_pixmap(widget->window,
+      gdk_draw_pixmap
 #endif
-		  widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
-		  pixmap_bal[mascot->bal_page],
-		  0,0,0,0,
-		  widget->allocation.width,
-		  widget->allocation.height);
-  
+      (widget->window,
+       widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
+       pixmap_bal[mascot->bal_page],
+       0,0,0,0,
+       widget->allocation.width,
+       widget->allocation.height);
   return FALSE;
-}
+}	   
 
 
 // ɥåˤư
@@ -1282,7 +1511,7 @@ void focus_out(GtkWidget * widget, GdkEventMotion * event)
 
 // ˥᡼󤪤ӻupdateѴؿ
 // timeoutʥŪ˸ƤӽФ
-gint time_update(gpointer gdata)
+gboolean time_update(gpointer gdata)
 {
   typMascot *mascot;
 #ifdef DEBUG
@@ -1297,7 +1526,7 @@ gint time_update(gpointer gdata)
   mascot=(typMascot *)gdata; 
 
   // ˥塼ФƤ
-  if(mascot->flag_menu) return;
+  if(mascot->flag_menu) return(TRUE);
 
   // Window Sitter
   if(mascot->move==MOVE_FOCUS){
@@ -1483,6 +1712,8 @@ gint time_update(gpointer gdata)
   if(mascot->raise_force) raise_all();
     
   gdk_flush();
+  
+  return(TRUE);
 
 }
 
@@ -1501,9 +1732,18 @@ void clock_update(typMascot *mascot, gboolean force_flag){
     caldigit=g_strdup_printf(_("%d/%d/%d(%s)"), tmpt->tm_year+1900,tmpt->tm_mon+1,
                                       tmpt->tm_mday,ini_week(tmpt->tm_wday));
 
+#ifdef __GTK_TOOLTIP_H__
+    gtk_widget_set_tooltip_text(clock_main,caldigit);
+#else
     gtk_tooltips_set_tip(tooltip,clock_main,caldigit,NULL);
+#endif
+
 #ifdef USE_WIN32
+#ifdef __GTK_TOOLTIP_H__
+    gtk_widget_set_tooltip_text(clock_fg,caldigit);
+#else
     gtk_tooltips_set_tip(tooltip,clock_fg,caldigit,NULL);
+#endif
 #endif
 
     g_free(caldigit);
@@ -1559,11 +1799,6 @@ void clock_update(typMascot *mascot, gboolean force_flag){
 	}
       }
     }
-  }
-  else{
-    // ư
-    pre_hour=tmpt->tm_hour;
-  }
 
   if(mascot->drag==FALSE){
     gboolean draw_flag=FALSE;
@@ -1579,9 +1814,10 @@ void clock_update(typMascot *mascot, gboolean force_flag){
 
     case CLOCK_TYPE_12S:
       if((tmpt->tm_sec!=pre_sec)||(force_flag)){
-	if(tmpt->tm_hour>12){
+	if(tmpt->tm_hour>=12){
 	  sprintf(mascot->digit,"%2d:%02d:%02d",
-		  tmpt->tm_hour-12,tmpt->tm_min,tmpt->tm_sec);
+		  tmpt->tm_hour==12?12:tmpt->tm_hour-12,tmpt->tm_min,
+		  tmpt->tm_sec);
 	  mascot->clk_pm=TRUE;
 	}
 	else{
@@ -1603,9 +1839,9 @@ void clock_update(typMascot *mascot, gboolean force_flag){
 
     case CLOCK_TYPE_12M:
       if((tmpt->tm_min!=pre_min)||(force_flag)){
-	if(tmpt->tm_hour>12){
+	if(tmpt->tm_hour>=12){
 	  sprintf(mascot->digit,"%2d:%02d",
-		  tmpt->tm_hour-12,tmpt->tm_min);
+		  tmpt->tm_hour==12?12:tmpt->tm_hour-12,tmpt->tm_min);
 	  mascot->clk_pm=TRUE;
 	}
 	else{
@@ -1632,208 +1868,25 @@ void clock_update(typMascot *mascot, gboolean force_flag){
 	// δؿWindowѹˤconfigure/expose_event
 	// ƤӽФ, pixmapwindowؤԤäƤ
 	if(force_flag){
-	  DrawPanelClock(mascot);
+	  DrawPanelClock0(mascot);
 	}
-	DrawPanelClock(mascot);
+	DrawPanelClock0(mascot);
 	break;
-	
       }
       pre_sec=tmpt->tm_sec;
       pre_min=tmpt->tm_min;
     }
   }
+
+  }
+  else{
+    // ư
+    pre_hour=tmpt->tm_hour;
+  }
+
 }
 
 
-// ѥͥפpixmap bufferؤ
-void DrawPanelClock(typMascot *mascot)
-{
-  gint clk_width,clk_height;
-  gint work_page;
-#ifdef USE_GTK2
-  PangoLayout *pango_text;
-#else
-  gint lb,rb,as,ds;
-#endif
-  gint new_w, new_h;
-
-  work_page=mascot->clk_page;
-  work_page^=1;
-
-#ifdef USE_GTK2
-  pango_text=gtk_widget_create_pango_layout(mascot->w_clkdraw,
-					    mascot->digit);
-  pango_layout_get_pixel_size(pango_text,&clk_width,&clk_height);
-#else
-  gdk_string_extents(mascot->fontclk,mascot->digit,
-		     &lb,&rb,&clk_width,&as,&ds);
-  clk_height=as+ds;
-#endif
-
-
-  switch(mascot->clktype){
-  case CLOCK_TYPE_12S:
-  case CLOCK_TYPE_12M:
-    clk_width+=clk_height/3;
-    break;
-  }
-
-  
-  if (pixmap_clk[work_page]) {
-#ifdef USE_GTK2
-    g_object_unref(G_OBJECT(pixmap_clk[work_page]));
-#else
-    gdk_pixmap_unref(pixmap_clk[work_page]);
-#endif
-  } 
-  
-  new_w=clk_width+(mascot->clktext_x+mascot->wclkbd)*2;
-  new_h=clk_height+(mascot->clktext_y+mascot->wclkbd)*2;
-  pixmap_clk[work_page] = gdk_pixmap_new(mascot->w_clkdraw->window,
-					 new_w,
-					 new_h,
-					 -1);
-  
-
-  // **** pixmapؤBUFFERING
-  // border
-  gdk_draw_rectangle(pixmap_clk[work_page],mascot->gc_clkbd[work_page],
-		     TRUE,
-		     0,0,
-		     clk_width+(mascot->clktext_x+mascot->wclkbd)*2,
-		     clk_height+(mascot->clktext_y+mascot->wclkbd)*2);
-
-  // background
-  gdk_draw_rectangle(pixmap_clk[work_page],mascot->gc_clkbg[work_page],
-		     TRUE,
-		     mascot->wclkbd,mascot->wclkbd,
-		     clk_width+mascot->clktext_x*2,
-		     clk_height+mascot->clktext_y*2);
-
-  
-  // digital clock
-  if(mascot->flag_clksd){
-#ifdef USE_GTK2
-    gdk_draw_layout(pixmap_clk[work_page],
-		    mascot->gc_clksd[work_page],
-		    mascot->clktext_x+mascot->wclkbd+1,
-		    mascot->clktext_y+mascot->wclkbd+1,
-		    pango_text);
-#else
-    gdk_draw_string(pixmap_clk[work_page],mascot->fontclk,
-		    mascot->gc_clksd[work_page],
-		    mascot->clktext_x+mascot->wclkbd+1,
-		    as+mascot->clktext_y+mascot->wclkbd+1,
-		    mascot->digit);
-#endif
-  }
-#ifdef USE_GTK2
-  gdk_draw_layout(pixmap_clk[work_page],
-		  mascot->gc_clk[work_page],
-		  mascot->clktext_x+mascot->wclkbd,
-		  mascot->clktext_y+mascot->wclkbd,
-		  pango_text);
-#else
-  gdk_draw_string(pixmap_clk[work_page],mascot->fontclk,
-		  mascot->gc_clk[work_page],
-		  mascot->clktext_x+mascot->wclkbd,
-		  as+mascot->clktext_y+mascot->wclkbd,
-		  mascot->digit);
-#endif
-
-  switch(mascot->clktype){
-  case CLOCK_TYPE_12S:
-  case CLOCK_TYPE_12M:
-    // digital clock
-    if(mascot->flag_clksd){
-      if(mascot->clk_pm){
-	gdk_draw_arc(pixmap_clk[work_page],mascot->gc_clksd[work_page],
-		     TRUE,
-		     clk_width+mascot->clktext_x+mascot->wclkbd+1-clk_height/3,
-		     mascot->clktext_y+mascot->wclkbd+1+clk_height*2/3,
-		     clk_height/3,clk_height/3,
-		     0,(360*64));
-      }
-      else{
-	gdk_draw_arc(pixmap_clk[work_page],mascot->gc_clksd[work_page],
-		     TRUE,
-		     clk_width+mascot->clktext_x+mascot->wclkbd+1-clk_height/3,
-		     mascot->clktext_y+mascot->wclkbd+1,
-		     clk_height/3,clk_height/3,
-		     0,(360*64));
-      }
-    }
-    
-    if(mascot->clk_pm){
-      gdk_draw_arc(pixmap_clk[work_page],mascot->gc_clk[work_page],
-		   TRUE,
-		   clk_width+mascot->clktext_x+mascot->wclkbd-clk_height/3,
-		   mascot->clktext_y+mascot->wclkbd+clk_height*2/3,
-		   clk_height/3,clk_height/3,
-		   0,(360*64));
-    }
-    else{
-      gdk_draw_arc(pixmap_clk[work_page],mascot->gc_clk[work_page],
-		   TRUE,
-		   clk_width+mascot->clktext_x+mascot->wclkbd-clk_height/3,
-		   mascot->clktext_y+mascot->wclkbd,
-		   clk_height/3,clk_height/3,
-		   0,(360*64));
-    }
-
-    break;
-  }
-
-  mascot->clk_page=work_page;
-  gdk_flush();
-
-
-#ifdef USE_WIN32
-  if((mascot->flag_clkfg)&&(mascot->alpha_clk!=100)){
-    gdk_window_set_back_pixmap(mascot->w_clkdraw_fg->window,
-			       pixmap_clk[mascot->clk_page],
-			       FALSE);
-  }
-#endif
-  gdk_window_set_back_pixmap(mascot->w_clkdraw->window,
-			     pixmap_clk[mascot->clk_page],
-			     FALSE);
-
-  if((mascot->w_clkdraw->allocation.width!=new_w)
-     ||(mascot->w_clkdraw->allocation.width!=new_h)){
-#ifdef USE_WIN32
-    if((mascot->flag_clkfg)&&(mascot->alpha_clk!=100)){
-      gtk_drawing_area_size (GTK_DRAWING_AREA(mascot->w_clkdraw_fg), 
-			     new_w,
-			     new_h);
-    }
-#endif
-
-    gtk_drawing_area_size (GTK_DRAWING_AREA(mascot->w_clkdraw), 
-			   new_w,
-			   new_h);
-
-#ifdef USE_WIN32
-    if((mascot->flag_clkfg)&&(mascot->alpha_clk!=100)){
-      gdk_window_resize (clock_fg->window,
-			 new_w,
-			 new_h);
-    }
-#endif
-    gdk_window_resize (clock_main->window,
-		       new_w,
-		       new_h);
-  }
-
-
-#ifdef USE_GTK2
-  g_object_unref(G_OBJECT(pango_text));
-#endif
-
-  while (my_main_iteration(FALSE));
-  gdk_flush();
-}
-
 
 gchar *ini_week(gint day_of_week){
   gchar * weekk=NULL;
@@ -1960,13 +2013,17 @@ void raise_all(void){
 	SWP_SHOWWINDOW|SWP_NOMOVE|SWP_NOSIZE);
   SetWindowLong(GDK_WINDOW_HWND(win_main->window), GWL_EXSTYLE, 
 	GetWindowLong(GDK_WINDOW_HWND(win_main->window), GWL_EXSTYLE)|WS_EX_NOACTIVATE);
+  gdk_window_raise(win_sdw->window);
+  gdk_window_raise(win_main->window);
 #else
   gdk_window_raise(win_main->window);
 #endif
+
 #ifdef USE_WIN32
   gdk_window_raise(clock_fg->window);
 #endif
   gdk_window_raise(clock_main->window);
+
 #ifdef USE_WIN32
   gdk_window_raise(balloon_fg->window);
 #endif
@@ -2003,10 +2060,13 @@ void callbacks_arg_init(){
 //*********************************************************
 //  Сΰζ֤
 //*********************************************************
-int GetTaskbarHeight( void )
+MyXY GetTaskbarHeight( void )
 {
   APPBARDATA abd;
   HWND hWndAutoHide;
+  MyXY xy;
+
+  xy.flag=TRUE;
   
   memset( &abd, 0, sizeof( abd ) );
   abd.cbSize = sizeof( abd );
@@ -2018,29 +2078,21 @@ int GetTaskbarHeight( void )
     SHAppBarMessage( ABM_GETTASKBARPOS, &abd );
   }
   if(abd.uEdge==ABE_BOTTOM){
-    return (abd.rc.bottom - abd.rc.top);
+    xy.y=(abd.rc.bottom - abd.rc.top);
   }
   else{
-    return(0);
+    xy.y=0;
   }	
-  
-}//GetTaskbarRect
 
-int GetTaskbarWidth( void )
-{
-  APPBARDATA abd;
-  
-  memset( &abd, 0, sizeof( abd ) );
-  abd.cbSize = sizeof( abd );
-  
-  SHAppBarMessage( ABM_GETTASKBARPOS, &abd );
+  //SHAppBarMessage( ABM_GETTASKBARPOS, &abd );
   if(abd.uEdge==ABE_BOTTOM){
-    return (abd.rc.right);
+    xy.x = (abd.rc.right);
   }
   else{
-    return(GetSystemMetrics(SM_CXVIRTUALSCREEN));
+    xy.x =(GetSystemMetrics(SM_CXVIRTUALSCREEN));
   }	
   
+  return(xy);
 }//GetTaskbarRect
 
 //*********************************************************
@@ -2060,3 +2112,132 @@ gboolean IsWin32Root( HWND hWnd, gint root_width, gint root_height )
 }//GetWindowWidth
 #endif
 
+#ifndef USE_WIN32
+//*********************************************************
+// X Window  ߤΤääƤWindow̾
+//*********************************************************
+gchar* Get_Window_Name(dpy, top)
+     Display *dpy;
+     Window top;
+{
+  Window *children, dummy;
+  unsigned int nchildren;
+  int i;
+  Window w=0;
+  char *window_name;
+  
+  if (XFetchName(dpy, top, &window_name)){
+    return(window_name);
+  }
+  
+  if (!XQueryTree(dpy, top, &dummy, &dummy, &children, &nchildren)){
+    return(NULL);
+  }
+  
+  for (i=0; i<nchildren; i++) {
+    if(window_name!=NULL) g_free(window_name);
+    window_name = g_strdup(Get_Window_Name(dpy, children[i]));
+    if (window_name!=NULL) break;
+  }
+  if (children) XFree ((char *)children);
+  
+  return(window_name);
+}
+
+//*********************************************************
+// X Window  ߤΤääƤWindowBar Size
+//*********************************************************
+gint Get_Window_Bar_Size(dpy, top)
+     Display *dpy;
+     Window top;
+{
+  Atom type, atom;
+  int format;
+  union {
+    gulong *prop;
+    guchar *prop_ch;
+  } prop = { NULL };
+  gulong nitems;
+  gulong bytes_after; 
+  int error, result;
+  gint bar_size;
+  GdkWMDecoration decor=0;
+
+  bar_size=0;
+
+  gdk_error_trap_push ();
+  type = None;
+
+  atom=XInternAtom(dpy,"_NET_FRAME_EXTENTS", False);
+  result=XGetWindowProperty(dpy, top,
+			    atom,
+			    0,  G_MAXLONG,  FALSE,
+			    XA_CARDINAL,
+			    &type, &format, &nitems,
+			    &bytes_after, &prop.prop_ch);
+  error = gdk_error_trap_pop ();
+
+  if (error || result != Success){
+    return(bar_size);
+  }
+
+  if(nitems==4){
+    bar_size=prop.prop[2];
+  }
+
+
+  if (prop.prop)
+    XFree (prop.prop);
+		       
+  return(bar_size);
+}
+
+
+
+//*********************************************************
+// X Window  Сαɸ֤
+//    Home_X  =  root_x - ret.x
+//    Home_y  =  root_y - ret.y
+//*********************************************************
+MyXY GetAutoHomePos(void){
+    GdkAtom atom, atom_ret;
+    gint format, length;
+    guchar *data;
+    gboolean ret;
+    MyXY xy;
+
+    atom = gdk_atom_intern ("_NET_WORKAREA", TRUE);
+    if (atom != GDK_NONE) {
+        ret = gdk_property_get (
+#ifdef USE_GTK2
+				gdk_get_default_root_window (),
+#else
+				GDK_ROOT_PARENT(),
+#endif
+				atom, GDK_NONE, 0, G_MAXLONG, FALSE,
+				&atom_ret, &format, &length, &data);
+        if (ret && format == 32 && length / sizeof(glong) >= 4) {
+	  //g_print ("format = %d, length = %d\n", format, length);
+	  //  g_print ("x = %d, y = %d\n",
+	  //	     ((glong*) data)[0], ((glong*) data)[1]);
+	  // g_print ("width = %d, height = %d\n",
+	  //	     ((glong*) data)[2], ((glong*) data)[3]);
+	  xy.flag=TRUE;
+	  xy.x=((glong*) data)[0] + ((glong*) data)[2] ;
+	  xy.y=((glong*) data)[3] + ((glong*) data)[1] ;
+	} else {
+	  xy.flag=FALSE;
+	  xy.x=0;
+	  xy.y=0 ;
+	}
+    }
+    else{
+      xy.flag=FALSE;
+      xy.x=0;
+      xy.y=0 ;
+    }
+
+    return(xy);
+}
+
+#endif
