@@ -27,7 +27,7 @@
 
 #define POP_FIELD_VALUE_LENGTH    1024
 #define POP_OK_STR   "+OK"
-#define POP_ERR_STR  "+ERR"
+#define POP_ERR_STR  "-ERR"
 #define POP_OK 0
 #define POP_ERROR 1
 #define POP_INVALID 2
