@@ -5,7 +5,7 @@
 //     mail.c
 //     Biff functions for MaCoPiX (POP3 /APOP and local spool)
 //    
-//                            Copyright 2002-2007  K.Chimari
+//                            Copyright 2002-2008  K.Chimari
 //                                     http://rosegray.sakura.ne.jp/
 //
 //
@@ -52,9 +52,9 @@ void biff_init();
 
 gint SetMailChecker();
 gint ResetMailChecker();
-gint MailChecker();
+gboolean MailChecker();
 
-GtkWidget * make_biff_pix();
+void make_biff_pix();
 void display_biff_balloon();
 void remap_biff_pix();
 
@@ -100,6 +100,13 @@ extern GtkWidget *biff_pix;
 extern typMascot *Mascot;
 extern gboolean flag_balloon;
 
+
+extern gint dw_configure_biff_pix();
+extern gint dw_expose_biff_pix();
+extern void biff_drag_begin();
+extern void biff_drag_end();
+extern void biff_window_motion();
+
 extern void conv_unmime_header();
 extern void conv_unmime_header_overwrite();
 
@@ -143,15 +150,34 @@ void biff_init(typMascot *mascot)
   mascot->mail.last_s=NULL;
   if(pop_froms) g_free(pop_froms);
   pop_froms=strbuf(NULL);
+#ifdef __GTK_TOOLTIP_H__
+  //gtk_widget_set_tooltip_text(mascot->mail.e_draw,
+  //gtk_widget_set_tooltip_text(mascot->mail.e_draw,
+  gtk_widget_set_tooltip_text(biff_pix,
+			      _("Initializing Biff..."));
+#else
   gtk_tooltips_set_tip(GTK_TOOLTIPS(mascot->mail.tooltips),
-		       mascot->mail.e_draw,
+		       //mascot->mail.e_draw,
+		       biff_pix,
 		       _("Initializing Biff..."),
 		       NULL);
+#endif
+
   if(mascot->mail.tooltips_fl){
+#ifdef __GTK_TOOLTIP_H__
+    //gtk_widget_set_tooltip_text(mascot->mail.e_draw,NULL);
+    gtk_widget_set_tooltip_text(biff_pix,NULL);
+#else
     gtk_tooltips_enable(mascot->mail.tooltips);
+#endif
   }
   else{
+#ifdef __GTK_TOOLTIP_H__
+    //gtk_widget_set_tooltip_text(mascot->mail.e_draw,NULL);
+    gtk_widget_set_tooltip_text(biff_pix,NULL);
+#else
     gtk_tooltips_disable(mascot->mail.tooltips);
+#endif
   }
   mascot->mail.pop_readed=FALSE;
 
@@ -181,7 +207,7 @@ gint SetMailChecker(gpointer gdata){
   mascot=(typMascot *)gdata;
 
   if(mascot->mail.proc_id>0){
-    gtk_timeout_remove((guint)mascot->mail.proc_id);
+    g_source_remove((guint)mascot->mail.proc_id);
     fflush(stdin);
     fflush(stdout);
   }
@@ -191,8 +217,9 @@ gint SetMailChecker(gpointer gdata){
     biff_init(mascot);
     MailChecker(mascot);
     mascot->mail.proc_id
-      =(gint)gtk_timeout_add((guint32)(mascot->mail.interval*1000),MailChecker,
-			     (gpointer)mascot);
+      =(gint)g_timeout_add((guint32)(mascot->mail.interval*1000),
+			   (GSourceFunc)MailChecker,
+			   (gpointer)mascot);
 
     pop_debug_print("proc_id=%d\n",mascot->mail.proc_id);
   }
@@ -208,7 +235,7 @@ gint ResetMailChecker(gpointer gdata){
   }
 
   if(mascot->mail.proc_id>0){
-    gtk_timeout_remove((guint)mascot->mail.proc_id);
+    g_source_remove((guint)mascot->mail.proc_id);
     fflush(stdin);
     fflush(stdout);
   }
@@ -217,8 +244,9 @@ gint ResetMailChecker(gpointer gdata){
   if(mascot->mail.flag){
     MailChecker(mascot);
     mascot->mail.proc_id
-      =(gint)gtk_timeout_add((guint32)(mascot->mail.interval*1000),MailChecker,
-			     (gpointer)mascot);
+      =(gint)g_timeout_add((guint32)(mascot->mail.interval*1000),
+			   (GSourceFunc)MailChecker,
+			   (gpointer)mascot);
 
     pop_debug_print("proc_id=%d\n",mascot->mail.proc_id);
   }
@@ -227,12 +255,12 @@ gint ResetMailChecker(gpointer gdata){
 }
 
 
-gint MailChecker(gpointer gdata){
+gboolean MailChecker(gpointer gdata){
   typMascot *mascot;
 
   mascot=(typMascot *)gdata;
 
-  if(mascot->flag_menu) return;
+  if(mascot->flag_menu) return(TRUE);
 
   gdk_flush();
 
@@ -245,13 +273,12 @@ gint MailChecker(gpointer gdata){
     display_biff_balloon(mascot);
 
   gdk_flush();
-
+  return(TRUE);
 }
 
 
-GtkWidget * make_biff_pix(typMascot *mascot){
-  GtkWidget *biff_drawing_area;
-
+void make_biff_pix(typMascot *mascot){
+  
   biff_pix = gtk_window_new(GTK_WINDOW_POPUP);
 #ifdef USE_GTK2
   gtk_window_set_accept_focus(GTK_WINDOW(biff_pix),FALSE);
@@ -264,24 +291,29 @@ GtkWidget * make_biff_pix(typMascot *mascot){
 			GDK_BUTTON_PRESS_MASK | 
 			GDK_EXPOSURE_MASK);
   gtk_widget_realize(biff_pix);
+#ifdef USE_GTK2
+  gtk_window_set_resizable(GTK_WINDOW(biff_pix),TRUE);
+#endif
   gdk_window_set_decorations(biff_pix->window, 0);
 #ifndef USE_WIN32
   /* gdk_window_set_override_redirect is not implemented (for warning) */
   gdk_window_set_override_redirect(biff_pix->window,TRUE);
 #endif
 
-  mascot->mail.e_draw=gtk_event_box_new();
-  gtk_container_add(GTK_CONTAINER (biff_pix), mascot->mail.e_draw);
-  gtk_widget_show(mascot->mail.e_draw);
- 
-  biff_drawing_area = gtk_drawing_area_new();
-  gtk_drawing_area_size (GTK_DRAWING_AREA(biff_drawing_area), 1, 1);
-  
-  gtk_container_add(GTK_CONTAINER (mascot->mail.e_draw), 
-		    biff_drawing_area);
-  gtk_widget_show(biff_drawing_area);
+  my_signal_connect(biff_pix, "configure_event",
+		    dw_configure_biff_pix, (gpointer)Mascot);
+  my_signal_connect(biff_pix, "expose_event",
+		    dw_expose_biff_pix, (gpointer)Mascot);
+  my_signal_connect(biff_pix, "button_press_event",
+		    biff_drag_begin, (gpointer)Mascot);
+  my_signal_connect(biff_pix, "button_release_event",
+		    biff_drag_end, (gpointer)Mascot);
+  my_signal_connect(biff_pix, "motion_notify_event",
+		    biff_window_motion, (gpointer)Mascot);
 
-  return(biff_drawing_area);
+#ifdef USE_GTK2
+  gtk_window_resize (GTK_WINDOW(biff_pix), 1, 1);
+#endif
 }
 
 
@@ -306,25 +338,46 @@ void display_biff_balloon(typMascot *mascot)
 				     mascot->mail.last_f,mascot->mail.last_s);
 	}
 	if(text_tip){
+#ifdef __GTK_TOOLTIP_H__
+	  gtk_widget_set_tooltip_text(biff_pix,
+				      //gtk_widget_set_tooltip_text(mascot->mail.e_draw,
+				      text_tip);
+#else
 	  gtk_tooltips_set_tip(GTK_TOOLTIPS(mascot->mail.tooltips),
-			       mascot->mail.e_draw,
+			       biff_pix,
+			       //mascot->mail.e_draw,
 			       text_tip,NULL);
+#endif
 	  g_free(text_tip);
 	}
       }
       else{
 	text_tip=g_strdup_printf(_("You have %d mails."),mascot->mail.count);
+#ifdef __GTK_TOOLTIP_H__
+	//	gtk_widget_set_tooltip_text(mascot->mail.e_draw,
+	gtk_widget_set_tooltip_text(biff_pix,
+				    text_tip);
+#else
 	gtk_tooltips_set_tip(GTK_TOOLTIPS(mascot->mail.tooltips),
-			     mascot->mail.e_draw,
+			     //mascot->mail.e_draw,
+			     biff_pix,
 			     text_tip,NULL);
+#endif
 	g_free(text_tip);
       }
     }
     else{
+#ifdef __GTK_TOOLTIP_H__
+      //gtk_widget_set_tooltip_text(mascot->mail.e_draw,
+      gtk_widget_set_tooltip_text(biff_pix,
+				  _("You have no mails."));
+#else
       gtk_tooltips_set_tip(GTK_TOOLTIPS(mascot->mail.tooltips),
-			   mascot->mail.e_draw,
+			   //mascot->mail.e_draw,
+			   biff_pix,
 			   _("You have no mails."),
 			   NULL);
+#endif
     }
   }
 
@@ -372,6 +425,7 @@ void display_biff_balloon(typMascot *mascot)
 
 
 void remap_biff_pix(typMascot *mascot){
+  if(!mascot->mail.flag) return;
   if(mascot->mail.status == (POP3_ERROR|POP3_SSL_CERT)){
     gtk_widget_unmap(biff_pix);
   }
@@ -1248,16 +1302,17 @@ gchar * fs_get_mbox(typMascot *mascot){
 	  if(g_strncasecmp(buf,"from:",5)==0)
 	    {
 	      strcpy(bufs,buf);
-	      for(i=0;i<10;i++){
+	      ///***for(i=0;i<10;i++){
+	      while(1){
 		fgetpos(fp,&pos);
 		if((p=fgets(buf,BUFFSIZE-1,fp))==NULL
 		   ||buf[0]=='\n'||buf[0]=='\0'){
 		  ed_fl=TRUE;
 /* tnaka start */
-if( strlen(bufs)) {
-	f = g_strdup(bufs);
-	fsetpos(fp,&pos);
-}
+		  if( strlen(bufs)) {
+		    f = g_strdup(bufs);
+		    fsetpos(fp,&pos);
+		  }
 /* tnaka end */
 		  break;
 		}
@@ -1281,16 +1336,17 @@ if( strlen(bufs)) {
 	  else if(g_strncasecmp(buf,"subject:",8)==0)
 	    {
 	      strcpy(bufs,buf);
-	      for(i=0;i<10;i++){
+	      ///***for(i=0;i<10;i++){
+	      while(1){
 		fgetpos(fp,&pos);
 		if((p=fgets(buf,BUFFSIZE-1,fp))==NULL
 		   ||buf[0]=='\n'||buf[0]=='\0'){
 		  ed_fl=TRUE;
 /* tnaka start */
-if( strlen(bufs)) {
-	s = g_strdup(bufs);
-	fsetpos(fp,&pos);
-}
+		  if( strlen(bufs)) {
+		    s = g_strdup(bufs);
+		    fsetpos(fp,&pos);
+		  }
 /* tnaka end */
 		  break;
 		}
@@ -1385,7 +1441,8 @@ gchar *  fs_get_procmail(typMascot  *mascot){
 		    if(g_strncasecmp(buf,"from:",5)==0)
 		    {
 			strcpy(bufs,buf);
-			for(i=0;i<10;i++){
+			///***for(i=0;i<10;i++){
+			while(1){
 			    fgetpos(fp_folder,&pos);
 			    if((p=fgets(buf,BUFFSIZE-1,fp_folder))==NULL
 			       ||buf[0]=='\n'||buf[0]=='\0'){
@@ -1411,7 +1468,8 @@ gchar *  fs_get_procmail(typMascot  *mascot){
 		    else if(g_strncasecmp(buf,"subject:",8)==0)
 		    {
 			strcpy(bufs,buf);
-			for(i=0;i<10;i++){
+			///***for(i=0;i<10;i++){
+			while(1){
 			    fgetpos(fp_folder,&pos);
 			    if((p=fgets(buf,BUFFSIZE-1,fp_folder))==NULL
 			       ||buf[0]=='\n'||buf[0]=='\0'){
@@ -1522,7 +1580,8 @@ gchar * fs_get_qmail(typMascot *mascot){
 		    if(g_strncasecmp(buf,"from:",5)==0)
                     {
 			strcpy(bufs,buf);
-			for(i=0;i<10;i++){
+			///***for(i=0;i<10;i++){
+			while(1){
 			    fgetpos(fp_folder,&pos);
 			    if((p=fgets(buf,BUFFSIZE-1,fp_folder))==NULL
 			       ||buf[0]=='\n'||buf[0]=='\0'){
@@ -1548,7 +1607,8 @@ gchar * fs_get_qmail(typMascot *mascot){
 		    else if(g_strncasecmp(buf,"subject:",8)==0)
                     {
 			strcpy(bufs,buf);
-			for(i=0;i<10;i++){
+			///***for(i=0;i<10;i++){
+			while(1){
 			    fgetpos(fp_folder,&pos);
 			    if((p=fgets(buf,BUFFSIZE-1,fp_folder))==NULL
 			       ||buf[0]=='\n'||buf[0]=='\0'){
@@ -1687,7 +1747,9 @@ void fs_get_pop3(int num, typMascot *mascot){
       {
 	strcpy(bufs, buffer_header[header]);
 	//bufs=g_strconcat(buffer_header[header],NULL);
-	for(i=1;i<10;i++){
+	///***for(i=1;i<10;i++){
+	i=1;
+	while(1){
 	  if((i+header) > header_line)  break;
 	  if((buffer_header[i+header][0]==' ') || (buffer_header[i+header][0]=='\t')){
 	    bufs[strlen(bufs)-2]='\0';
@@ -1699,13 +1761,16 @@ void fs_get_pop3(int num, typMascot *mascot){
 	    f=g_strdup(bufs);
 	    break;
 	  }
+	  i++;
 	}
       }
       else if(g_strncasecmp(buffer_header[header],"subject:",8)==0)
       {
 	strcpy(bufs, buffer_header[header]);
 	//bufs=g_strconcat(buffer_header[header],NULL);
-	for(i=1;i<10;i++){
+	///***for(i=1;i<10;i++){
+	i=1;
+	while(1){
 	  if((i+header) > header_line)  break;
 	  if((buffer_header[i+header][0]==' ') || (buffer_header[i+header][0]=='\t')){
 	    bufs[strlen(bufs)-2]='\0';
@@ -1717,6 +1782,7 @@ void fs_get_pop3(int num, typMascot *mascot){
 	    s=g_strdup(bufs);
 	    break;
 	  }
+	  i++;
 	}
       }
     }
