@@ -5,7 +5,7 @@
 //      main.h  
 //      Configuration header  for  MaCoPiX
 //
-//                            Copyright 2002-2007  K.Chimari
+//                            Copyright 2002-2008  K.Chimari
 //                                     http://rosegray.sakura.ne.jp/
 //
 //
@@ -23,6 +23,55 @@
 //  along with this program; if not, write to the Free Software
 //  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 //
+//
+//  difference between Gtk+2 version
+//
+//  - Translucent mascot and biff images
+//       gdk_window_set_opacity
+//       Gtk+ 2.12
+//  
+//  - Tooltip for clock  (__GTK_TOOLTIP_H__)
+//       Gtk+2.12
+//     If no, use Gtktooltips.
+//
+//  - Auto detection of composited or not 
+//      gtk_widget_is_composited
+//       Gtk+2.10
+//     If no, flag_composite=COMPISTE_UNKNOWN
+//
+//  - Text layout for Cairo (__PANGOCAIRO_H__)
+//      pango/pangocairo.h
+//       Gtk+2.10
+//     If no, use cairo_show_text.
+//
+//  - Icon on GNOME system tray  (__GTK_STATUS_ICON_H__)
+//       gtkstatusicon.h
+//       Gtk+2.10 
+//
+//  - Translucent Clock and Balloon (USE_CAIRO)
+//       cairo/cairo.h
+//       Gtk+2.8??
+//
+//  - Combo Box    (__GTK_COMBO_BOX__)
+//       gtkcombobox.h
+//       Gtk+ 2.4 
+//     If no, use GtkCombo.
+//
+//  - Color Button    (__GTK_CLOR_BUTTON__)
+//       gtkcolorbutton.h
+//       Gtk+ 2.4 
+//     If no, use GtkColorSelection and GtkButton.
+//
+//  - File Chooser    (__GTK_FILE_CHOOSER__)
+//       gtkfilechooser.h
+//       Gtk+ 2.4 
+//     If no, use GtkFileSelection.
+//
+//  - Icon on menus    (__GTK_STOCK_H__)
+//       gtkstock.h
+//       Gtk+ 2.2 (Some Items could be replaced by Gtk+ version.)
+//
+//
 
 
 #ifndef MAIN_H
@@ -45,6 +94,16 @@
 #include<gdk-pixbuf/gdk-pixbuf.h>
 //#include <gdk_imlib.h>
 
+#ifndef DISABLE_CAIRO
+#if defined(CAIRO_H)
+#define USE_CAIRO
+#endif
+#endif
+
+#ifdef USE_CAIRO
+#include <cairo.h>
+#endif
+
 #include "version.h"
 #include "configfile.h"
 #include "intl.h"
@@ -155,10 +214,9 @@ enum{ FOLDER_DEFAULT,
 
 // CLOCK_MODE
 enum{ CLOCK_NO,
-	CLOCK_PIXMAP,
-	CLOCK_PANEL
-	}ClockMode;
-
+      CLOCK_PIXMAP,
+      CLOCK_PANEL
+}ClockMode;
 
 //ANIME
 #define MAX_PIXMAP   64
@@ -181,6 +239,7 @@ enum{ CLOCK_NO,
 #define INIT_CLK_TEXT   5
 #define INIT_CLK_BORDER 2
 #endif
+#define INIT_CLK_SD   1
 
 #define INIT_BAL_TEXT   3
 #define INIT_BAL_BORDER 1
@@ -204,6 +263,8 @@ enum{
 // Focus Follow 
 enum{ FF_SIDE_LEFT, FF_SIDE_RIGHT } FFPos;
 
+// Focus Autobar ȥСˡ
+enum{ AUTOBAR_MANUAL, AUTOBAR_ORDINAL, AUTOBAR_COMPIZ } AutoBar;
 
 
 // Balloon Position
@@ -264,6 +325,11 @@ enum{ CLOCK_TYPE_24S,
 	CLOCK_TYPE_12M } ClockType;
 
 
+// Font size ratio for AM/PM sign
+#ifdef USE_CAIRO
+#define CLOCK_AMPM_RATIO 0.6
+#endif
+
 // Install mode
 enum{   MENU_SELECT, 
 	  MENU_INSTALL_USER,
@@ -273,16 +339,45 @@ enum{   MENU_SELECT,
 	  START_MENU_INSTALL_COMMON,
  } MenuSelect;
 
-#ifdef USE_WIN32
+#if GTK_CHECK_VERSION(2,12,0) || defined(USE_CAIRO) || defined(USE_WIN32)
 //DEFAULT Alpha
 #define DEF_ALPHA_MAIN 100
 #ifdef USE_BIFF
 #define DEF_ALPHA_BIFF 100
 #endif
+#endif
+
+#ifdef USE_WIN32
 #define DEF_ALPHA_BAL 80
 #define DEF_ALPHA_CLK 50
 #endif
 
+#ifdef USE_CAIRO
+#define CAIRO_DEF_ALPHA_OTHER 0xFFFF
+#define CAIRO_DEF_ALPHA_SDW 0xB000
+#ifdef USE_WIN32
+#define CAIRO_DEF_ALPHA_CLK 0xFFFF
+#define CAIRO_DEF_ALPHA_BAL 0xFFFF
+#else
+#define CAIRO_DEF_ALPHA_CLK 0x9000
+#define CAIRO_DEF_ALPHA_BAL 0xB000
+#endif
+#endif
+
+#ifdef USE_CAIRO // SHADOW
+
+#ifdef USE_WIN32
+#define CAIRO_SHADOW_X 2.0
+#define CAIRO_SHADOW_Y 2.0
+#else
+#define CAIRO_SHADOW_X 2.3
+#define CAIRO_SHADOW_Y 2.3
+#endif
+
+#define CAIRO_SHADOW_ALPHA 40
+#endif
+
+
 //DEFAULT COLOR
 #ifdef USE_WIN32
 #define COLOR_CLK_R 0xffff
@@ -441,6 +536,14 @@ typedef enum {
 } SSLCertRes;
 
 
+// COMPOSITE_FLAG
+enum{ COMPOSITE_FALSE,
+      COMPOSITE_TRUE,
+      COMPOSITE_UNKNOWN
+}CompositeFlag;
+
+
+
 
 
 // Ѵؿ
@@ -470,9 +573,19 @@ typedef struct{
   gchar *filename;
   GdkPixmap *pixmap;
   GdkBitmap *mask;
-
+#ifdef USE_WIN32
+  GdkPixmap *pixmap_sdw;
+  GdkBitmap *mask_sdw;
+#endif
 }typSprite;
 
+// դĤglong¤Ρ(x,y) (w,h)ʤ
+typedef struct{
+  glong x;
+  glong y;
+  gboolean flag;
+}MyXY;
+
 // 빽¤
 typedef struct _typCursor typCursor;
 struct _typCursor{
@@ -492,6 +605,17 @@ struct _typSignal{
   gboolean flag;
 };
 
+#ifdef USE_CAIRO
+// PangoCairo FontѴ
+typedef struct _myPangoCairo myPangoCairo;
+struct _myPangoCairo{
+  gchar *family;
+  gdouble pointsize;
+  cairo_font_slant_t slant;
+  cairo_font_weight_t weight;
+};
+#endif
+
 // ˥塼ѹ¤
 typedef struct _typScanMenu typScanMenu;
 struct _typScanMenu{
@@ -536,9 +660,11 @@ struct _typMail{
   gint proc_id;
   GdkPixmap *pixmap;
   GdkBitmap *mask;
-  GtkWidget *w_draw;
-  GtkWidget *e_draw;
+  //GtkWidget *w_draw;
+  //GtkWidget *e_draw;
+#ifndef __GTK_TOOLTIP_H__
   GtkTooltips *tooltips;
+#endif
   gboolean  tooltips_fl;
   gboolean drag;
   gchar *fs_max;
@@ -576,6 +702,9 @@ struct _typMascot{
   int yfix;
   int height;
   int width;
+#ifdef USE_WIN32
+  int sdw_height;
+#endif
   gint magnify;
   gint ip_style;
   int height_root;
@@ -597,12 +726,18 @@ struct _typMascot{
   gchar *fontname_clk;  
   gchar *deffontname_bal;  
   gchar *deffontname_clk;  
+#ifdef USE_CAIRO
+  myPangoCairo fontbal_pc;
+  myPangoCairo fontclk_pc;
+#endif
   typSprite *sprites;
   int clkmode;
   int clk_x;
   int clk_y;
   int clktext_x;
   int clktext_y;
+  gint clksd_x;
+  gint clksd_y;
   int wclkbd;
   int wbalbd;
   int baltext_x;
@@ -610,21 +745,14 @@ struct _typMascot{
   int balseq;
   gint balwidth;
   gint balheight;
-  GtkWidget *w_drawing;
-  GtkWidget *w_clkdraw;
-#ifdef USE_WIN32
-  GtkWidget *w_clkdraw_fg;
-#endif
-  GtkWidget *w_baldraw;
-#ifdef USE_WIN32
-  GtkWidget *w_baldraw_fg;
-#endif
+  //GtkWidget *w_drawing;
   GdkGC *gc_main[2];
   GdkGC *gc_mainsd[2];
   GdkGC *gc_clk[2];
   GdkGC *gc_clksd[2];
   GdkGC *gc_clkbg[2];
   GdkGC *gc_clkbd[2];
+  GdkGC *gc_clkmask[2];
   GdkGC *gc_bal[2];
   GdkGC *gc_balbg[2];
   GdkGC *gc_balbd[2];
@@ -643,7 +771,21 @@ struct _typMascot{
   GdkColor *colbal;
   GdkColor *colbalbg;
   GdkColor *colbalbd;
-#ifdef USE_WIN32
+#ifdef USE_CAIRO
+  gint def_alpclk;
+  gint def_alpclksd;
+  gint def_alpclkbg;
+  gint def_alpclkbd;
+  gint def_alpbal;
+  gint def_alpbalbg;
+  gint def_alpbalbd;
+  gint alpclk;
+  gint alpclksd;
+  gint alpclkbg;
+  gint alpclkbd;
+  gint alpbal;
+  gint alpbalbg;
+  gint alpbalbd;
   gboolean force_def_alpha;
   gint alpha_main;
   gint def_alpha_main;
@@ -651,12 +793,20 @@ struct _typMascot{
   gint alpha_biff;
   gint def_alpha_biff;
 #endif
+#endif
+#ifdef USE_WIN32
   gint alpha_bal;
   gint def_alpha_bal;
   gint alpha_clk;
   gint def_alpha_clk;
+#endif
+#ifdef USE_CAIRO
+  gboolean flag_img_cairo;
+  gboolean flag_bal_cairo;
+  gboolean flag_clk_cairo;
 #endif
   gboolean flag_clksd;
+  gboolean flag_clkrd;
 #ifdef USE_WIN32
   gboolean def_flag_clkfg;
   gboolean flag_clkfg;
@@ -686,17 +836,21 @@ struct _typMascot{
   int click_total;
   gint move;
   gboolean raise_force;
-  gboolean focus_autobar;
+  gboolean raise_kwin;
+  guint focus_autobar;
 #ifdef USE_WIN32
   gboolean task_force;
-  gboolean no_capbar;
 #endif
+  gboolean no_capbar;
   gint bar_size;
   gint bar_offset;
   gint xoff;
   gint yoff;
   gint homepos_nf;
   gint homepos_out;
+  gint homepos_nb;
+  gboolean home_auto;
+  gboolean home_auto_vanish;
   gint home_x;
   gint home_y;
   gint ff_side;
@@ -752,6 +906,18 @@ struct _typMascot{
   GtkStatusIcon *tray_icon;
   gboolean tray_icon_flag;
 #endif
+#if GTK_CHECK_VERSION(2,12,0) || defined(USE_CAIRO)
+  gint flag_composite;
+  gint force_composite;
+#endif
+#ifdef USE_CAIRO
+  gint    sdw_flag;
+  gfloat  sdw_x;
+  gfloat  sdw_y;
+  gint    sdw_alpha;
+#endif
+  gint sdw_x_int;
+  gint sdw_y_int;
 };
 
 #endif
