@@ -5,7 +5,7 @@
 //      sockmsg.c
 //      Management of Socket message function
 //
-//                            Copyright 2002-2007  K.Chimari
+//                            Copyright 2002-2008  K.Chimari
 //                                     http://rosegray.sakura.ne.jp/
 //
 //
@@ -77,7 +77,8 @@ gchar *cut_spc(gchar * obj_name){
         ||(tgt_name[i_bak]==0x0A)
         ||(tgt_name[i_bak]==0x0D)
         ||(tgt_name[i_bak]==0x09)){
-    tgt_name[i_bak]=(char)NULL;
+    //tgt_name[i_bak]=(char)NULL;
+    tgt_name[i_bak]=0x00;
     i_bak--;
   }
                                                                                 
