@@ -65,6 +65,10 @@ void my_signal_connect();
 extern gchar* to_utf8();
 extern gchar* to_locale();
 
+#ifndef USE_WIN32
+gchar* GetCurrentWMName();
+#endif
+gdouble GetCurrentResolution();
 
 void copy_file(gchar *src, gchar *dest)
 {
@@ -373,3 +377,22 @@ void my_signal_connect(GtkWidget *widget,
 #endif
 }
 
+
+#if defined(USE_GTK2) && !defined(USE_WIN32)
+gchar* GetCurrentWMName(GtkWidget *widget){
+  GdkScreen *screen = gtk_widget_get_screen(widget);
+  //GdkScreen *screen = gdk_screen_get_default();
+
+  return((gchar *)gdk_x11_screen_get_window_manager_name(screen));
+}
+#endif
+
+gdouble GetCurrentResolution(void){
+#if GTK_CHECK_VERSION(2,10,0) 
+  GdkScreen *screen = gdk_screen_get_default();
+
+  return(gdk_screen_get_resolution(screen));
+#else
+  return(96.0);
+#endif
+}
